package components.UIComponents
{
	import components.baseComponents.ExtendedDateField;
	
	import evenements.Action;
	
	import flash.events.Event;
	
	import gestionnaires.EventHandlerControler;
	
	import interfaces.IActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.containers.HBox;
	import mx.controls.Spacer;
	import mx.events.CalendarLayoutChangeEvent;

	/** Cette classe implémente un composant utilisé pour sélectionner une date et une heure.
	 *  Ce composant peut diffuser les actions suivantes :
	 *  o TimeChanged
	 *  o DateChanged
	 * 
	 *  Note: Les actions de type "TimeChanged" sont envoyées par les "sous composants" de type "TimePicker".
	 */

	public class DateTimePicker extends HBox implements IGhostInterface, ISelectorInterface, IActorInterface
	{
		private var __datePicker:ExtendedDateField;
		private var __timePicker:TimePicker;
		private var __enabled:Boolean              = new Boolean();
		private var __evtHd:EventHandlerControler  = new EventHandlerControler();
		private var __myName:String                = null;
		private var __css:Object                   = null;
		
		/** Construit un objet de sélection de la date et de l'heure.
		 *  @param enabled Cette valeur détermine l'état initial du composant.
		 *         Les valeurs possibles sont :
		 *         * true 
		 *         * false
		 *         Par défaut, le composant est actif.
		 *  @param fromDate Objet de type "Date". Cet objet représente la date de "début" du calendrier (exclue). La sélection est impossible avant cette date (y compris!).
		 *         Par défaut, cette date n'est pas définie.
		 *  @param toDate Objet de type "Date". Cet objet représente la date de "fin" du calendrier. La sélection est impossible après cette date (y compris!).
		 *         Par défaut, cette date n'est pas définie.
		 *  @param time L'heure d'initialisation. Deux formats sont utilisables :
		 *              - Un nombre entier compris entre 0 et 1439 (inclus), qui représente le nombre de minutes écoulées depuis 0h.
		 *              - Une chaîne "HH:MM".  
		 *  @param timeStep Taille de la marche entre deux variations du sélecteur d'heure.
		 *  @param language Cette chaîne de caractères définit la langue utilisée pour l'affichage de la date.
		 *         Les valeurs possibles sont :
		 *         * french
		 *         * english
		 *         La langue utilisée par défaut est l'anglais.
		 *  @param inCSS Cet objet contient les noms des classes CSS à appliquer.
		 *         Les propriétés sont les suivantes :
	 	 *         o 'clockText':            Nom de la classe CSS qui détermine l'apparence du texte de l'horloge.
	 	 *         o 'clockButton':          Nom de la classe CSS qui détermine l'apparence des boutons de l'horloge.
	 	 *         o 'clockBox':             Nom de la classe CSS qui détermine l'apparence du conteneur de l'horloge.
	 	 *         o 'dateClockContainer':   Nom de la classe CSS qui détermine l'apparence du conteneur.
		 *         o 'dateClockCalendar':    Nom de la classe CSS qui détermine l'apparence du calendrier.
		 */
		
		public function DateTimePicker(enabled:Boolean=true,
		                               fromDate:Date=null,
		                               toDate:Date=null,
		                               time:String="0",
		                               timeStep:Number=1,
		                               language:String='english',
		                               inCss:Object=null)
		{
			super();
			__enabled  = enabled;
			__css      = inCss; trace("DateTimePicker\n" + __css.toString());  
			
			
			
			// Création des composants.
			__datePicker = new ExtendedDateField();
			__timePicker = new TimePicker(10, "0", __css);
			
			// Initialisation de l'heure du composant.
			__timePicker.setValues({time:time, step:timeStep});
			
			// Configuration du calendrier.
			__datePicker.yearNavigationEnabled = false;
			__datePicker.showToday             = true;
			if (language == 'french')
			{
				__datePicker.dayNames   = new Array('D', 'L', 'M', 'M', 'J', 'V', 'S');
				__datePicker.monthNames = new Array('Janvier', 'Février', 'Mars', 'Avril', 'Mai', 'Juin', 'Juillet', 'Août', 'Septembre', 'Octobre', 'Novembre', 'Décembre');
			}
			if ((fromDate != null) || (toDate   != null)) { __datePicker.disabledRanges = []; }
			if (fromDate != null)     { __datePicker.disabledRanges.push({rangeEnd: fromDate}); }
			if (toDate   != null)     { __datePicker.disabledRanges.push({rangeStart: toDate}); }
			if (! __enabled) { disable(); }
			
			__createTimeChooser();
			__setEventHandler();
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant. 
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;		
			__timePicker.setName(inName);
		}
		
		/** Cette méthode est utilisée pour déclarer les évènements traités par cette classe.
		 */		
		
		private function __setEventHandler():void
		{
			__evtHd.addEventHandler(__datePicker, CalendarLayoutChangeEvent.CHANGE, __dateUpdated);
		}
		
		/** Crée le composant utilisé pour sélectionner une heure.
		 */
		
		private function __createTimeChooser():void
		{
			var dateHourSpacer:Spacer = new Spacer();
			
			// Séparateur entre de composant de sélection de la date et celui de l'heure.
			dateHourSpacer.width         = 5;
			dateHourSpacer.percentHeight = 100;
			
			// Le sélecteur de l'heure est désactivé à la création du composant.
			// On suppose que l'heure n'est ajustable que si l'on a, au préalable, choisi une date.
			__timePicker.disable();
			
			// Application des CSS.
			__setCss();
			
			// Assemblage			
			this.horizontalScrollPolicy = "off";
			this.verticalScrollPolicy   = "off";
			this.setStyle("verticalAlign", "middle");
			this.addChild(__datePicker);
			this.addChild(dateHourSpacer);
			this.addChild(__timePicker);
		}
		
		/**
		 * Cette méthode applique les CSS au composant.
		 */
		
		private function __setCss():void
		{
			if (__css != null)
			{
				if (__css.hasOwnProperty('dateClockContainer')) { this.styleName         = __css.dateClockContainer; }
				if (__css.hasOwnProperty('dateClockCalendar'))  { __datePicker.styleName = __css.dateClockCalendar; }
				
				
			}
		}
		
		/** Cette méthode rend le composant éligible à la destruction.
		 */
		
		public function destroy():void
		{
			__evtHd.removeAllEventHandlers();
			__timePicker.destroy();
		}
		
		/** Cette méthode renvoie la date sélectionnée.
		 *  La date retournée ne comporte pas d'information sur l'heure (HH=00, MM=00, SS=00).
		 *  @return Cette méthode renvoie la date sélectionnée.
		 *          If no date has been selected, the function returns the value null.
		 */
		
		public function selectedDate():Date { return __datePicker.selectedDate; }

		/** Cette méthode renvoie la date et l'heure sélectionnée.
		 *  @return Cette méthode renvoie la date et l'heure sélectionnée.
		 *          If no date has been selected, the function returns the value null.
		 */
		
		public function selectedDateTime():Date
		{
			var o:Object;

			if (__datePicker.selectedDate == null) { return null; }
			o = getSelectedValues();
			trace ('selectedDateTime() : year:' + o.year + ' month:' + o.month + ' day:' + o.date + ' hour:' + o.hour + ' minute:' + o.minute);   
			return new Date(o.year, o.month, o.date, o.hour, o.minute, 0, 0);
		}
		
		/** Cette méthode renvoie les valeurs sélectionnées par l'utilisateur.
		 *  @return La méthode renvoie un objet qui contient deux propriétés :
		 *          * hour    // L'heure.
		 *          * minute  // Le nombre de minutes depuis le début de l'heure.
		 *          * value   // Le nombre total de minutes depuis 0h.
		 *          * date    // 1 à 31
		 *          * day     // 0 pour Sunday, 1 pour Monday
		 *          * year    // The year
		 *          * month   // 0 pour January, 1 pour February
		 *          If no date has been selected, the function returns the value null.
		 */
		
		public function getSelectedValues():Object
		{
			var values:Object;
			
			if (__datePicker.selectedDate == null) { return null; }
			values = __timePicker.getSelectedValues();
			values.date  = __datePicker.selectedDate.date;
			values.day   = __datePicker.selectedDate.day;
			values.year  = __datePicker.selectedDate.fullYear;
			values.month = __datePicker.selectedDate.month;
			return values;
		}
		
		/** Cette fonction renvoie uniquement l'heure sélectionnée, sous la forme d'une chaîne de caractères ("HH:MM").
		 *  @return La fonction renvoie uniquement l'heure sélectionnée, sous la forme d'une chaîne de caractères ("HH:MM").
		 *          If no date has been selected, the function returns the value null.
		 */
		
		public function getTimeAsString():String
		{
			if (__datePicker.selectedDate == null) { return null; }
			return __timePicker.getSelectedValues().hour + ':' + __timePicker.getSelectedValues().minute; 
		}
		
		/** Cette méthode permet d'initialiser un composant de sélection avec des valeurs.
		 *  @param inValues Cet objet contient les valeurs utilisées pour l'initialisation du composant.
		 *         L'objet passé en argument de la méthode peut contenir les propriétés suivantes :
		 *         * time: Valeur utilisée pour initialiser l'horloge. Deux formats sont utilisables :
		 *                 - Un nombre entier compris entre 0 et 1439 (inclus), qui représente le nombre de minutes écoulées depuis 0h.
		 *                 - Une chaîne "HH:MM". 
		 *         * step: Valeur de la marche entre deux valeurs successives des minutes.
		 *         * date: Objet de type Date.
		 *         * bagin: Date de bébut de l'interval de sélection.
		 *         * end: Date de fin de l'interval de sélection.
		 *  @param inDispatchEvt Cet argument permet de spécifier si le composant doit envoyer un évènement suite à la mise à jour de l'heure.
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			__timePicker.setValues(inValues, inDispatchEvt);
			if (inValues.hasOwnProperty('date'))  { __datePicker.selectedDate = inValues.date; }
			if (inValues.hasOwnProperty('begin')) { __datePicker.enableRangeStart(inValues.begin); }
			if (inValues.hasOwnProperty('end'))   { __datePicker.enableRangeEnd(inValues.end); }
		}
		
		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			__datePicker.enabled = true;
			__timePicker.enable();
		}
		
		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			__datePicker.enabled = false;
			__timePicker.disable();
		}

		/** Cette méthode réinitialise le composant.
		 *  @param inValues Cet objet spécifie la liste des valeurs à réinitialiser.
		 *         Chaque propriété de l'objet représente une propriété à réinitialiser.
		 *         Les propriétés possibles sont :
		 *         o time
		 *         * step
		 *         o date
		 */
		
		public function reset(inValues:Object=null):void
		{
			if (inValues == null)
			{
				__datePicker.selectedDate = null;
				__timePicker.reset();
				__disableTimePicket();
				return;
			}
			
			if (inValues.hasOwnProperty('time'))	{
														__timePicker.reset(inValues);
														// trace ('DateTimePicker(' + __myName  + '): time cleared.');
													}
			if (inValues.hasOwnProperty('date'))	{
														__datePicker.selectedDate = null;
														__disableTimePicket();
														// trace ('DateTimePicker(' + __myName  + '): Text cleared.');
													}
		}
		
		/** Cette méthode désactive le sélecteur de l'heure.
		 */
		
		private function __disableTimePicket():void
		{
			__timePicker.setTime('00:00');
			__timePicker.disable();
		}
		
		/** Ce gestionnaire d'évènement est exécuté lorsque la date sélectionnée sur le calendrier est modifiée.
		 *  @param event Evènement reçut.
		 */
		
		private function __dateUpdated(event:Event):void
		{
			// Il est important de ne pas exécuter cette action avant que le composant ne soit complètement initialisé.
			// Avant que le composant ne soit complètement initialisé, l'appel à la méthode "setName()" ne peut pas avoir été effectué.
			// Le troisième paramètre du constructeur de l'évènement doit valoir "true".
			
			if (initialized)
			{
				if (__myName === null) { throw new Error('The component has no name.'); }
				dispatchEvent(new Action('Action', __myName, "DateChanged", true));
				trace ('DateTimePicker(' + __myName + '): DateChanged');
				__timePicker.enable();
			}
		}
	}
}