package components.UIComponents
{
	import evenements.Action;
	
	import flash.events.Event;
	
	import gestionnaires.EventHandlerControler;
	
	import interfaces.IActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.containers.HBox;
	import mx.containers.VBox;
	import mx.controls.Button;
	import mx.controls.Spacer;
	import mx.controls.Text;
	import mx.events.FlexEvent;

	/** Cette classe implémente un composant qui permet de sélectionner une heure à la façon d'un "NumericStepper".
	 *  Ce composant peut diffuser les actions suivantes :
	 *  o TimeChanged
	 */

	public class TimePicker extends HBox implements IGhostInterface, ISelectorInterface, IActorInterface
	{
		private var __clockScreen:Text            = new Text();
		private var __upButton:Button             = new Button();
		private var __downButton:Button           = new Button();
		private var __evtHd:EventHandlerControler = new EventHandlerControler();
		private var __myName:String;
		private var __currentStep:Number = 0;    // Numéro de la marche courante. Ce nombre croit, ou décroît, par unité de 1.
		private var __stepSize:Number    = 1;    // Taille de la marche.
		private var __upperStep:Number   = 0;    // Numéro de la marche la plus haute (limite supérieure de __currentStep).
		private var __css:Object         = null;
		
		/** construit un objet de type "TimePicker".
		 *  @param inStepSize Valeur de la marche entre deux valeurs successives des minutes.
		 *  @param inTime Valeur utilisée pour initialiser l'horloge. Deux formats sont utilisables :
		 *         o Un nombre entier compris entre 0 et 1439 (inclus), qui représente le nombre de minutes écoulées depuis 0h.
		 *         o Une chaîne "HH:MM".
		 *  @param inCSS Cet objet contient les noms des classes CSS à appliquer.
	 	 *         Les propriétés sont les suivantes :
	 	 *         o 'clockText':   Nom de la classe CSS qui détermine l'apparence du texte de l'horloge.
	 	 *         o 'clockButton': Nom de la classe CSS qui détermine l'apparence des boutons de l'horloge.
	 	 *         o 'clockBox':    Nom de la classe CSS qui détermine l'apparence du conteneur.
  		 */
		
		public function TimePicker(inStepSize:Number=1,
		                           inTime:String="0",
		                           inCss:Object=null)
		{
			super();
			__css = inCss; trace ("TimePicker\n" + __css.clockText);
			__createTimePicker();
			__setEventHandler();
			setStepSize(inStepSize);
			setTime(inTime);
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant. 
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;			
		}
		
		/** Ajuste l'heure de l'horloge.
		 *  @param inTime Valeur utilisée pour initialiser l'horloge. Deux formats sont utilisables :
		 *         o Un nombre entier compris entre 0 et 1439 (inclus), qui représente le nombre de minutes écoulées depuis 0h.
		 *         o Une chaîne "HH:MM".
		 *  @exception Si la valeur transmise à la fonction n'est pas correcte, alors une exception est lancée.
		 */
		
		public function setTime(inTime:String):void
		{
			var result:Object;
			var value:Number;
			var mod:Number;
			var regExpLiteral:RegExp = /^\s*(\d{1,2})\s*:\s*(\d{1,2})\s*$/;
			var regExpNumeric:RegExp = /^\s*(\d+)\s*$/;
			
			result = regExpLiteral.exec(inTime);
			if (result != null) { value = Number(result[1])*60 + Number(result[2]); } 
			else	{
						result = regExpNumeric.exec(inTime);
						if (result != null) { value = Number(result[1]); }
						else { throw new Error('Invalid time value: (' + inTime + ')'); } 
					}
			
			if (value >= 1440) { throw new Error('Time value out of range: ' + inTime); } 
			
			mod = value % __stepSize;
			__currentStep = (value - mod) / __stepSize;
			__format();
		}
		
		/** Ajuste la valeur de la marche entre deux valeurs successives des minutes.
		 *  @param inStepSize Valeur de la marche entre deux valeurs successives des minutes.
		 */
		
		public function setStepSize(inStepSize:Number):void
		{
			__stepSize  = inStepSize;
			__upperStep = ((1440 - (1440 % inStepSize)) / inStepSize) - 1;
		}
		
		/** Cette fonction permet d'ajuster l'horloge.
		 *  @param inValues Cet objet contient les paramètres d'ajustage. Les valeurs possibles sont les suivantes :
		 *         o time
		 *         o step
		 *         Veuillez vous référer à la documentation du constructeur afin d'obtenir la signification de ces paramètres.
		 *  @param inDispatchEvt Cet argument permet de spécifier si le composant doit envoyer un évènement suite à la mise à jour de l'heure.
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			var action:Boolean = false;
			if (inValues.hasOwnProperty('step')) { setStepSize(inValues.step); action = true; }
			if (inValues.hasOwnProperty('time')) { setTime(inValues.time);     action = true; }
			if (inDispatchEvt && action) { __dispatchEvent(); }
		}
		
		/** Cette méthode permet de récupérer l'heure sélectionnée par l'utilisateur.
		 *  @return La méthode renvoie un objet qui présente les propriétés suivantes :
		 *          o hour: Le nombre d'heures depuis 0h00.
		 *          o minute: Le nombre de minutes depuis le début de l'heure.
		 *          o value: Le nombre de minutes total depuis 0h00.
		 */
		
		public function getSelectedValues():Object
		{
			var minutes:Number = 0;
			var hour:Number    = 0;
			var minute:Number  = 0;
			
			minutes = __currentStep * __stepSize;
			minute  = minutes % 60;
			hour    = (minutes - minute) / 60;
			return {hour:hour, minute:minute, value:minutes}; 
		}
		
		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			__upButton.enabled   = true;
			__downButton.enabled = true;
		}
		
		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			__upButton.enabled   = false;
			__downButton.enabled = false;
		}

		/** Cette méthode réinitialise le composant.
		 *  @param inValues Cet objet spécifie la liste des valeurs à réinitialiser.
		 *         La liste des propriétés attendues est la suivante :
		 *           o time
		 *           o step
		 *         Veuillez vous référer à la documentation du constructeur afin d'obtenir la signification de ces paramètres.
		 */
		
		public function reset(inValues:Object=null):void
		{
			if (inValues == null) { return; }
			setValues(inValues);
		}
		
		/** Cette méthode est utilisée pour déclarer les évènements traités par cette classe.
		 */		
		
		private function __setEventHandler():void
		{
			__evtHd.addEventHandler(__upButton,   FlexEvent.BUTTON_DOWN, __forwardClock);
			__evtHd.addEventHandler(__downButton, FlexEvent.BUTTON_DOWN, __rewindClock);
		}
		
		/** Cette méthode rend l'objet éligible à la destruction.
		 */
		
		public function destroy():void
		{
			__evtHd.removeAllEventHandlers();
		}
		
		/** Cette méthode crée l'interface utilisateur.
		 */
		
		private function __createTimePicker():void
		{
			var buttonVBox:VBox         = new VBox();
			var buttonUpSpacer:Spacer   = new Spacer();
			var buttonDownSpacer:Spacer = new Spacer();
			
			// Zone d'affichage de l'heure.
			__clockScreen.width = 60;
			__clockScreen.setStyle("textAlign", "center");
			// __clockScreen.setStyle("fontSize", "15");
			// __clockScreen.htmlText = '<b>00:00</b>';
			
			// Positionnement des boutons de réglage de l'heure.
			buttonUpSpacer.percentHeight   = 100;
			buttonUpSpacer.percentWidth    = 100;
			buttonDownSpacer.percentHeight = 100;
			buttonDownSpacer.percentWidth  = 100;
			__upButton.autoRepeat          = true;
			__upButton.height              = 13;
			__upButton.width               = 20;
			__downButton.autoRepeat        = true;
			__downButton.height            = 13;
			__downButton.width             = 20;
			buttonVBox.setStyle('paddingBottom', 0);
			buttonVBox.setStyle('paddingTop', 0);
			buttonVBox.setStyle('verticalGap', 0);
			buttonVBox.addChild(buttonUpSpacer);
			buttonVBox.addChild(__upButton);
			buttonVBox.addChild(__downButton);
			buttonVBox.addChild(buttonDownSpacer);
			
			// Allication des CSS
			__setCss();
			
			this.setStyle('verticalAlign', 'middle');
			this.addChild(__clockScreen);
			this.addChild(buttonVBox);
		}
		
		/**
		 * Cette méthode applique les CSS au composant.
		 */
		
		private function __setCss():void
		{
			if (__css != null)
			{
				if (__css.hasOwnProperty('clockText'))  	{ __clockScreen.styleName = __css.clockText; }
				if (__css.hasOwnProperty('clockButton'))	{ 
																__upButton.styleName   = __css.clockButton;
																__downButton.styleName = __css.clockButton;
															}
				if (__css.hasOwnProperty('clockBox'))    	{ this.styleName          = __css.clockBox; }
			}
		}
		
		/** Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur avance l'heure de l'horloge.
		 *  @param event Evènement reçut.
		 */
		
		private function __forwardClock(event:Event):void
		{
			if (__currentStep >= __upperStep) { return; }
			__currentStep += 1;
			__format();
			__dispatchEvent();
		}

		/** Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur recule l'heure de l'horloge.
		 *  @param event Evènement reçut.
		 */
		
		private function __rewindClock(event:Event):void
		{
			if (__currentStep == 0) { return; }
			__currentStep -= 1;
			__format();
			__dispatchEvent();
		}
		
		/** Cette méthode formate la chaîne de caractères affichée par l'horloge.
		 */
		
		private function __format():void
		{
			var data:Object = getSelectedValues();
			__clockScreen.htmlText = '<b>' + __numberToString(data.hour)  + ':' + __numberToString(data.minute) + '</b>';
		}

		/** Cette fonction envoie l'évènement qui indique que l'heure a été modifiée.
		 */
		
		private function __dispatchEvent():void
		{
			// Il est important de ne pas exécuter cette action avant que le composant ne soit complètement initialisé.
			// Avant que le composant ne soit complètement initialisé, l'appel à la méthode "setName()" ne peut pas avoir été effectué.
			// Le troisième paramètre du constructeur de l'évènement doit valoir "true".

			if (initialized)
			{	
				if (__myName === null) { throw new Error('The component has no name.'); }
				dispatchEvent(new Action('Action', __myName, "TimeChanged", true));
				// trace ('TimePicker(' + __myName + '): TimeChanged');
			}
		}
		
		/** Retourne une chaîne de caractères qui représente l'heure, ou les minutes, sur deux caractères.
		 *  @param n Nombre entier qui représente l'heure ou les minutes.
		 */
		
		private function __numberToString(n:Number):String
		{
			if (n < 10) { return '0' + n.toString(); }
			return n.toString();
		}
		
	}
}