package components.baseComponents
{
	import evenements.Action;
	
	import flash.events.Event;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.collections.XMLListCollection;
	import mx.controls.ComboBox;
	import mx.events.FlexEvent;
	import mx.events.ListEvent;
	
	import tools.DataProviderTools;
	import tools.FunctionUtilities;

	/** Ce composant simplifie l'utilisation de la liste déroulante de sélection.
	 *  Pour être associée à ce composant, la liste des items doit se présenter de la façon suivante :
	 * 
	 *  <?xml version="1.0" encoding="UTF-8"?>
	 *  <root>
	 *		<item label="..." data="..."/>
	 *		<item label="..." data="..."/>
	 *		<item label="..." data="..."/>
	 *		...
	 *  </root>
	 * 
	 *  @remark Le nom de la balise racine (ici "root") n'a pas d'importante.
	 *  @remark Ce composant ne devrait pas être utilisé pour afficher une liste de sélection téléchargée depuis le WEB.
	 *          En effet, il existe un composant adapté à cette opération : HttpComboBox.
	 *  @remark Ce composant est susceptible de lancer les évènements suivants :
	 *          o DataSetted
	 *          o DataChanged
	 */

	public class ExtendedComboBox extends ComboBox implements IGhostInterface, IActorInterface, ICompositeActorInterface, ISelectorInterface
	{
		private var __myName:String;
		private var __onModifHd:Function = tools.FunctionUtilities.doNothing;
		
		public function ExtendedComboBox()
		{
			super();
			this.dataProvider = null;
			this.addEventListener(ListEvent.CHANGE, __dataChanged);
		}
		
		/** Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur modifie l'item sélectionné dans la liste de sélection déroulante.
		 *  @param even Cet évènement est envoyé lorsque l'utilisateur modifie l'item sélectionné dans la liste de sélection déroulante.
		 */
		
		private function __dataChanged(event:Event):void
		{
			__onModifHd(getSelectedValues());
			this.dispatchEvent(new Action('Action', __myName, 'DataChanged', true));
		}
		
		/** Cette méthode libère les ressources allouées au composant.
		 */
		
		public function destroy():void
		{
			this.removeEventListener(FlexEvent.DATA_CHANGE, __dataChanged);
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant.
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;
		}
		
		/** Cette méthode permet de définir une fonction, qui sera exécutée par l'acteur, à chaque fois que ce dernier change d'état.
		 *  @param inFunction Fonction exécutée par le composant à chaque fois que ce dernier change d'état.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction != null) { __onModifHd = inFunction; }
		}
		
		/** Cette méthode renvoie les valeurs sélectionnées via le composant.
		 *  @return La méthode renvoie un objet qui présente les propriétés suivantes :
		 *          o data
		 *          o item
		 */
		
		public function getSelectedValues():Object
		{
			if (this.dataProvider == null) { throw new Error("Empty data provider!"); }

			return { data:this.selectedItem.@data, item:this.selectedItem };
		}
		
		/** Cette méthode permet d'initialiser le composant.
		 *  Cette initialisation concerne la liste d'item sélectionnable ou l'item sélectionné.
		 *  @param inValues Cet objet contient les valeurs utilisées pour réinitialiser le composant. Les propriétés de l'objet sont les suivantes :
		 *         o list: Si cette propriété est définie, alors sa valeur est utilisée pour définir la liste des items sélectionnables.
		 *         o itemData: Si cette propriété est définie, alors sa valeur est utilisée pour positionner l'item sélectionné, à partir de sa valeur.
		 *         o itemIndex: Si cette propriété est définie, alors sa valeur est utilisée pour positionner l'item sélectionné, à partir de son index.
		 *  @param inDispatchEvt Cet argument permet de spécifier si le composant doit envoyer un évènement suite à son initialisation.
		 *  @remark Le composant admet deux façons de désigner l'item sélectionné :
		 *          o En spécifiant sa valeur, définie par l'attribut "data".
		 *          o En spécifiant l'index de l'item dans la liste des items.
		 *          Si les deux façons de spécifier l'item courant sont spécifiées simultanément (par valeur ou par index), c'est la première (par valeur) qui est utilisée.
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			if ((! inValues.hasOwnProperty('list')) && (this.dataProvider == null))
			{ throw new Error('The combo box must be associated to a data provider before any other action!'); }
			
			if (inValues.hasOwnProperty('list'))
			{
				this.dataProvider = new XMLListCollection(inValues.list.item);
				this.labelField   = '@label';
			}
			
			if (inValues.hasOwnProperty('itemData'))
			{
				this.selectedIndex = tools.DataProviderTools.getComboBoxDataProviderIndex('data', inValues.itemData, this.dataProvider);
			}
			else { if (inValues.hasOwnProperty('itemIndex')) { this.selectedIndex = inValues.itemIndex; } }
			
			if (inDispatchEvt) { this.dispatchEvent(new Action('Action', __myName, 'DataSetted', true)); }
		}
		
		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			this.enabled = true;
		}
		
		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			this.enabled = false;
		}
		
		/** Cette méthode permet de réinitialiser le composant.
		 *  Cette réinitialisation concerne la liste d'items sélectionnables, ou l'item sélectionné.
		 *  @param inValues Cet objet contient les valeurs utilisées pour réinitialiser le composant. Les propriétés de l'objet sont les suivantes :
		 *         o 'list': Si cette propriété est définie, alors sa valeur est utilisée pour définir la liste des items sélectionnables.
		 *         o 'itemData': Si cette propriété est définie, alors sa valeur est utilisée pour positionner l'item sélectionné, à partir de sa valeur.
		 *         o 'itemIndex': Si cette propriété est définie, alors sa valeur est utilisée pour positionner l'item sélectionné, à partir de son index.
		 *         Si cet argument vaut la valeur, alors la liste d'items sélectionnée est vidée.
		 */
		
		public function reset(inValues:Object=null):void
		{
			if (inValues == null) { this.dataProvider = null; return; }
			setValues(inValues, false);
		}
		
	}
}