package components.baseComponents
{
	import evenements.Action;
	
	import flash.events.Event;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.controls.RichTextEditor;
	import mx.events.FlexEvent;
	
	import tools.FunctionUtilities;
	
	/** Cette classe étend le composant d'édition élaboré de Flex.
	 *  Ce composant est susceptible d'envoyer des actions de types :
	 *  o DataSetted
	 *  o TextChanged
	 */

	public class ExtendedRichTextEditor extends RichTextEditor implements IActorInterface, ICompositeActorInterface, IGhostInterface, ISelectorInterface
	{
		private var __onModifHd:Function = tools.FunctionUtilities.doNothing;
		private var __myName:String;
		
		/** Cette méthode construit une instance d'éditeur.
		 */
		
		public function ExtendedRichTextEditor()
		{
			this.addEventListener(flash.events.Event.CHANGE,        __textChanged);
			this.addEventListener(mx.events.FlexEvent.VALUE_COMMIT, __textChanged);
		}

		/** Ce gestionnaire d'évènement est exécuté lorsque le contenu de l'éditeur change.
		 *  @param event Évènement qui signale un changement de contenu de l'éditeur.
		 */

		private function __textChanged(event:Event):void
		{
			__onModifHd(getSelectedValues());
			dispatchEvent(new Action('Action', __myName, "TextChanged", true));
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant. 
		 */

		public function setName(inName:String):void
		{
			__myName = inName;
		}
		
		/** Cette méthode permet de déclarer une fonction qui sera automatiquement exécutée à chaque modification du contenu de l'éditeur.
		 *  @param inFunction Fonction à exécuter.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction != null) { __onModifHd = inFunction }
		}
		
		/** Cette méthode rend le composant éligible à la destruction.
		 */
		
		public function destroy():void
		{
			this.removeEventListener(flash.events.Event.CHANGE,        __textChanged);
			this.removeEventListener(mx.events.FlexEvent.VALUE_COMMIT, __textChanged);
		}
		
		/** Cette méthode renvoie le texte entré par l'utilisateur.
		 *  @return La méthode renvoie un objet qui présente deux propriétés :
		 *          o 'text': Le contenu de l'éditeur, sans aucun formatage.
		 *          o 'html': Le contenu de l'éditeur, avec un formatage HTML.
		 */
		
		public function getSelectedValues():Object
		{
			return { text:this.text, html:this.htmlText };
		}
		
		/** Cette méthode permet de remplir la zone d'édition avec du texte.
		 *  @param inValues Cet objet contient le texte à injecter dans l'éditeur.
		 *         Deux formats de texte sont pris en charge :
		 *         o 'text': Texte brut.
		 *         o 'html': HTML.
		 *         Seul un type de format est accepté. Si les deux sont présents, alors le format HTML sera utilisé.
		 *  @param inDispatchEvt Cet indicateur permet de spécifier si l'on désire envoyer un évènement ou non lors de l'initialisation du composant.
		 *         La valeur va entraîner l'envoi d'un évènement de type "DataSetted".
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			if (inValues.hasOwnProperty('html')) { this.htmlText = inValues.html; }
			else { if (inValues.hasOwnProperty('text')) { this.text = inValues.text; } }
			
			if (inDispatchEvt) { dispatchEvent(new Action('Action', __myName, "DataSetted", true)); }
		}
		
		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			this.enabled = true;	
		}
		
		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			this.enabled = false;
		}
		
		/** Cette méthode réinitialise le composant.
		 *  @param inValues Cet objet spécifie la liste des valeurs à réinitialiser.
		 *         Chaque propriété de l'objet représente une propriété à réinitialiser.
		 */
		
		public function reset(inValues:Object=null):void
		{
			this.text = '';
		}
		
	}
}