package components.baseComponents
{
	import evenements.Action;
	
	import flash.events.Event;
	
	import gestionnaires.EventHandlerControler;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.collections.XMLListCollection;
	import mx.controls.ComboBox;
	import mx.events.FlexEvent;
	import mx.events.ListEvent;
	
	import tools.DataProviderTools;
	import tools.FunctionUtilities;
	import tools.HttpLoader;

	/** Cette classe implémente une boîte de sélection dont les paramètres nécessaires à son initialisation sont récupérés via HTTP.
	 *  Pour être associé à une composant de type "ComboBox", le document téléchargé doit présenter la structure suivante :
	 * 
	 *  <?xml version="1.0" encoding="UTF-8"?>
	 *  <root>
	 *		<item label="..." data="..."/>
	 *		<item label="..." data="..."/>
	 *		<item label="..." data="..."/>
	 *		...
	 *  </root>
	 * 
	 *  Ce composant est susceptible de diffuser les évènements suivants :
	 *  o DataSelected - Cela signifie que le contenu du composant de sélection a été modifié (peu importe la nouvelle valeur sélectionnée).
	 *  o DataSetted - Cet évènement signifie que l'action d'initialisation de la valeur sélectionnée a été effectuée.	
	 * 
	 *  @remark Le nom de la balise racine (ici "root") n'a pas d'importante.
	 *  @remark Le nom de la balise qui définit les entrées (ici "item") est configurable (CF méthode setValues()).
	 *  @remark Le nom de la propriété qui définit le texte affiché sur chaque ligne de la boîte de sélection (ici "label") est configurable (CF méthode setValues()).
	 *  @remark Le nom de la propriété qui définit la valeur sélectionnée (ici "data") n'a pas d'importance (CF méthode setValues()).
	 *  @remark Si le document d'origine ne se présente cette structure, ce n'est pas un problème.
	 *          L'utilisateur de cette classe est invité à déclarer un greffon destiné à convertir le document téléchargé avant son utilisation.
	 *  @remark L'utilisateur peut également définir un greffon qui sera exécuté après l'injection des données téléchargées dans le composant de sélection.
	 * 
	 *  @warning Le chargement des informations d'initialisation du composant ne peut être effectué que lorsque l'application est entièrement initialisée!
	 *           Cette contrainte est justifiée par le fait que le composant lance un popup.
	 */

	public class HttpComboBox extends ComboBox implements IActorInterface, IGhostInterface, ISelectorInterface, ICompositeActorInterface
	{
		private var __evtHd:EventHandlerControler = new EventHandlerControler();
		private var __userPrePlugin:Function      = null;
		private var __userPostPlugin:Function     = null;
		private var __selectedData:String         = null;
		private var __onModifHd:Function          = tools.FunctionUtilities.doNothing;
		private var __values:Object;
		private var __myName:String;
		private var __http:HttpLoader;
		
		/** Crée une boîte de sélection dont les paramètres d'initialisation sont récupérés via HTTP.
		 *  @param inURL URL vers les paramètres d'initialisation de la boîte de sélection.
		 */
		
		public function HttpComboBox()
		{
			super();
			__http = new HttpLoader(this);
			__setEventHandler();
		}
		
		/** Cette méthode charge les données qui permettront d'initialiser le composant.
		 *  @param inValues Cet objet contient les valeurs utilisées pour initialiser le composant.
		 *         Ces valeurs sont:
		 *         o url: URl vers le document (ou le service WEB).
		 *         o plugin: Référence vers une fonction dont le rôle est de convertir le document téléchargé en un format compatible avec l'utilisation du composant "ComboBox".
		 *                   Cette fonction va également effectuer des opérations sur la boîte de sélection, une fois cette dernière initialisée.
		 *         o format: Ce paramètre indique le format vers lequel le document téléchargé doit être converti (CF doc FLEX HTTPService.resultFormat).
		 *         o method: "GET" ou "POST".
		 *         o values: Objet qui contient les valeurs à envoyer au service WEB (si l'URL pointe vers un service WEB).
		 *                   Si aucune valeur ne doit être envoyée, cette valeur doit être null.
		 *         o item: Nom de la balise qui contient les valeurs stockées dans le composant.
		 *         o label: Nom de l'attribut qui représente le texte associé à la valeur.
		 *         o data: Nom de l'attribut qui représente la valeur de l'élément sélectionné.
		 * 
		 *  @remark: Pour être associé à une composant de type "ComboBox", le document téléchargé doit présenter la structure suivante :
	 	 * 
		 *  <?xml version="1.0" encoding="UTF-8"?>
		 *  <root>
		 *		<item label="..." data="..."/>
		 *		<item label="..." data="..."/>
		 *		<item label="..." data="..."/>
		 *		...
		 *  </root>
		 */
		
		private function __loadData(inValues:Object):void
		{
			__http.load(inValues.url, inValues.plugin, inValues.format, true, inValues.values, inValues.method);
			// trace ("HttpCombo: Data loadded");
			// trace ("HttpCombo: " + __http.error());
		}
		
		/** Cette méthode est utilisée pour déclarer les évènements traités par cette classe.
		 */		
		
		private function __setEventHandler():void
		{
			__evtHd.addEventHandler(this, ListEvent.CHANGE,                 __comboClosed);
			__evtHd.addEventHandler(this, mx.events.FlexEvent.VALUE_COMMIT, __comboClosed);
		}
		
		/** Ce gestionnaire d'évènement est exécuté lorsque la boîte de sélection est refermée.
		 *  @param event Évènement envoyé lorsque la boîte de sélection se referme.
		 */
		
		private function __comboClosed(event:Event):void
		{
			var selectedData:String;
			
			// Il est important de ne pas exécuter cette action avant que le composant ne soit complètement initialisé.
			// Avant que le composant ne soit complètement initialisé, l'appel à la méthode "setName()" ne peut pas avoir été effectué.
			// Le troisième paramètre du constructeur de l'évènement doit valoir "true".
			
			if (initialized)
			{
				if (__myName === null) { throw new Error('The component has no name.'); }
				dispatchEvent(new Action('Action', __myName, "DataSelected", true));
				__onModifHd(getSelectedValues());
			}
			// trace ('__comboClosed()');
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant. 
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;
		}
		
		/** Cette méthode rend le composant éligible à la destruction.
		 */
		
		public function destroy():void
		{
			__evtHd.removeAllEventHandlers();
			__http.destroy();
		}
		
		/** Cette méthode renvoie les valeurs sélectionnées par l'utilisateur.
		 *  @return La méthode renvoie un objet qui contient les valeurs sélectionnées.
		 *          La valeur sélectionnée est représentée par la propriété "value" de l'objet retourné.
		 *          La valeur retournée peut être égale à null si aucun item n'a été sélectionné.
		 */
		
		public function getSelectedValues():Object
		{
			var item:XML;
			
			if (this.selectedItem == null) { return { value:null }; }
			
			item = XML(this.selectedItem);
			// trace(item.attribute(__values.data));
			return { value:item.attribute(__values.data).toString() };
		}
		
		/** Cette méthode permet d'initialiser un composant de sélection avec des valeurs.
		 *  @param inValues Cet objet contient les valeurs utilisées pour initialiser le composant.
		 *         o url: URl vers le document (ou le service WEB).
		 *         o prePlugin: Référence vers une fonction dont le rôle est de convertir le document téléchargé en un format compatible avec l'utilisation du composant "ComboBox".
		 *		   o postPlugin: Référence vers une fonction qui sera exécutée lorsque le document aura été totalement téléchargé. 
		 *         o format: Ce paramètre indique le format vers lequel le document téléchargé doit être converti (CF doc FLEX HTTPService.resultFormat).
		 *         o method: "GET" ou "POST".
		 *         o values: Objet qui contient les valeurs à envoyer au service WEB (si l'URL pointe vers un service WEB).
		 *           Si aucune valeur ne doit être envoyée, cette valeur doit être null.
		 *         o item: Nom de la balise qui contient les valeurs stockées dans le composant.
		 *         o label: Nom de l'attribut qui représente le texte associé à la valeur.
		 *         o data: Nom de l'attribut qui représente la valeur de l'élément sélectionné.
		 *         o selectedData: Si elle est définie, cette valeur représente la valeur présélectionnée de la liste de sélection.
		 *  @param inDisparchEvt Cet argument permet de spécifier si le composant doit envoyer un évènement suite à la mise à jour de l'heure.
		 * 
		 *  @warning ATTENTION!!! Cette méthode déclenche l'apparition d'une fenêtre surgissante.
		 *           La fenêtre surgissante requiert une fenêtre parente. On ne peut donc pas exécuter cette fonction tant que l'application n'est pas complètement initialisée!
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			// Chargement des valeurs d'initialisation.
			// Attention : Le chargement des documents est une opération asynchrone!
			// L'exécution de la fonction se poursuit alors que le document n'a pas encore été téléchargé!
			// C'est la raison pour laquelle nous passons un greffon (__httpPlugin) qui sera exécuté lorsque le document aura été téléchargé.
			// Ce greffon (__httpPlugin) va appeler les fonctions définies par l'utilisateur.

			__values          = inValues;
			__values.dispatch = inDispatchEvt;
			__userPrePlugin   = inValues.prePlugin;
			__userPostPlugin  = inValues.postPlugin;
			inValues.plugin   = __httpPlugin;
			__loadData(inValues);
		}
		
		/** Cette méthode constitue un greffon qui est passé à l'objet chargé de télécharger les données d'initialisation vie HTTP.
		 *  Si (et seulement si) les données d'initialisation sont correctement téléchargées, alors ce greffon est exécuté.
		 *  Ce greffon met à jour la boîte de sélection.
		 *  @param inValue Ce paramètre contiendra le document téléchargé par le composant de téléchargement.
		 *  @return La fonction renvoie la transformation du document téléchargé.
		 * 
		 *  @remark Il serait également possible de renvoyer la valeur null.
		 *  @remark Par l'intermédiaire des greffons passés en paramètres de la fonction "setValues()", ce code lance l'exécution du code spécifique, fournit par l'utilisateur.
		 */
		 
		private function __httpPlugin(inValue:Object):Object
		{
			var doc:Object;
			
			// Conversion du document téléchargé.
			doc = __userPrePlugin(inValue); 
			
			// Mise à jour du composant de sélection.
			this.dataProvider = new XMLListCollection(doc.elements(__values.item));
			this.labelField   = '@' + String(__values.label);
			
			// Association éventuelle de la valeur de présélection.
			if (__values.hasOwnProperty('selectedData')) { this.selectedIndex = tools.DataProviderTools.getComboBoxDataProviderIndex(__values.data, __values.selectedData, this.dataProvider); } 

			// Exécution du code spécifique fournit par l'utilisateur.
			if (__userPostPlugin != null) { __userPostPlugin(doc); }
			
			// Lance un éventuel évènement pour signaler que l'assignation de valeur a été effectuée.
			if (__values.dispatch) { dispatchEvent(new Action('Action', __myName, "DataSetted", true)); }
			
			return doc;		
		}
		
		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void { this.enabled = true; }

		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void { this.enabled = false; }
		
		/** Cette méthode réinitialise le composant.
		 *  @param inValues Cet objet spécifie la liste des valeurs à réinitialiser.
		 *         Chaque propriété de l'objet représente une propriété à réinitialiser.
		 */
		
		public function reset(inValues:Object=null):void
		{
			this.dataProvider = null;
		}
		
		/** Cette méthode permet de définir une fonction, qui sera exécutée par l'acteur, à chaque fois que ce dernier change d'état.
		 *  @param inFunction Fonction exécutée par le composant à chaque fois que ce dernier change d'état.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction == null) { return; }
 			__onModifHd = inFunction;
		}
		
	}
}