package components.compositComponents
{
	import components.popups.PopupMessage;
	import components.popups.WaitBox;
	
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.collections.XMLListCollection;
	import mx.containers.HBox;
	import mx.containers.VBox;
	import mx.controls.DataGrid;
	import mx.controls.Label;
	import mx.controls.LinkButton;
	import mx.controls.Spacer;
	import mx.core.IFlexDisplayObject;
	import mx.managers.PopUpManager;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.remoting.mxml.RemoteObject;
	
	import tools.FunctionUtilities;
	
	import values.Configuration;
	
	/**
	 * Séquence d'utilisation :
	 * o Construction du composant.
	 * o Personnalisation : Label de sélection de la page précédente.
	 *                      Label de sélection de la page suivante.
	 *                      Représentation de la progrssion de la pagination.
	 * o setValues().
	 * 
	 * ATTENTION !!!
	 * 
	 * Il faut bien comprendre que ce composant est (presque totalement) indépendant de l'interface présentée par le serveur avec qui il dialogue.
	 * Il est donc nécessaire de fournir au composant les procédures nécessaires à la manipulation des données reçues et envoyées.
	 * Ces procédures permettent d'extraire de la réponse du serveur certaines informations indispensables.
	 * Et, en complément, ces procédures permettent de forger les requêtes envoyées au serveur.
	 * 
	 * Le serveur (AMF) doit renvoyer un objet qui présente les propriétés suivantes :
	 *    o 'data' (Array): Tableau associatif qui contient les valeurs à visualiser.
	 *                      Ce tableau sera passé à la fonction __downloadedDataToXml() dont le rôle sera de le transformer afin de rendre l'information compatible avec le DataGrid.
	 *                      Ce tableau sera passé à la fonction __downloadedDataToControl() dont le rôle sera d'extraire de la réponse du serveur les informations nécessaires à la pagination.
	 *    o 'method' (String): Le nom de la méthode qui a été exécutée.
	 *    o 'status' (int): Status de l'opération (1: succès et 0: erreur).
	 *
	 * L'objet envoyé par le composant au serveur est entièrement construit par l'application FLEX (qui utilise le composant).
	 * Il incombe donc à l'application de construire cet objet en fonction de l'interface du serveur (AMF).
	 * Il incombe donc aussi à l'application de fournir au composant la procédure à suivre pour modifier la requête de telle façon d'avancer ou de reculer dans la sélection.
	 * La méthode __uploadRequestChangePage() décrit justement la procédure à suivre pour modifier la requête de telle façon d'avancer ou de reculer dans la sélection.
	 * 
	 * STYLES:
	 * o gridBrowserNavigationControlCSS
	 * o gridBrowserRankLabelCSS
	 * o gridBrowserFormularCSS
	 */	 

	public class DataGridBrowser extends VBox implements IActorInterface, ICompositeActorInterface, IGhostInterface, ISelectorInterface
	{
		/** Composants interactifs. */
		private var	__dataGrid:DataGrid;  
		private var	__prevLinkUp:LinkButton;
		private var __nextLinkUp:LinkButton;
		private var	__prevLinkDown:LinkButton;
		private var __nextLinkDown:LinkButton;
		private var __pageRankUp:Label;
		private var __pageRankDown:Label;
		private var __popup:IFlexDisplayObject;
		
		/** Organisation de la grille de visualisation. */
		private var __downloadedDataToXml:Function        = null;
		private var __downloadedDataToControl:Function    = null;
		private var __downloadedDataToPagination:Function = __defaultPageRank;
		private var __uploadRequestChangePage:Function    = null;
		private var __onSucessAction:Function             = tools.FunctionUtilities.doNothing;
		private var __columns:Array                       = null;
		private var __provider:XMLListCollection          = null;
		
		/** Lien avec le serveur. */
		private var __connector:RemoteObject;
		
		/** Éléments d'interaction avec le reste de l'application. */
		private var __serverUrl:String;
		private var __onModifHd:Function   = tools.FunctionUtilities.doNothing; 
		private var __myName:String        = null;
		private var __where:Object         = null;
		private var __noDataLabel:String   = null;
		private var __noSearchLabel:String = null;
		
		/** Données de contrôle. */
		private var __currentPage:int = 1;
		private var __totalPage:int   = -1;
		private var __loaded:Boolean  = false;


		/** Constructeur.
		 *  @param inServerUrl URL vers le serveur qui traite les requêtes de navigation.
		 *  @param inDownloadedDataToXml Cette fonction est utilisée par le présent composant pour transformer les informations en provenance du serveur afin de les rendre compatibles avec l'utilisation du DataGrid.
		 *         La signature de la fonction doit être :
		 *         function downloadedDataToXml(inRecords:Array):XMLListCollection
		 *         La valeur renvoyée sera utilisée pour renseigner le DataProvider au DataGrid.
		 *         Les attributs des éléments XML seront utilisés pour définir le contenu des colonnes.
		 *  @param inDownloadedDataToControl Cette fonction est utilisée pour extraire les éléments de contrôle envoyés à l'application FLEX par le serveur.
		 *         La signature de la fonction doit être :
		 *         function inDownloadedDataToControl(inData:Object):Array
		 *         L'objet renvoyé doit contenir les clés suivantes :
		 *         o 'currentPage': Le numéro de la page couramment affichée.
		 *         o 'totalPages': Le nombre total de pages.
		 *  @param inUploadRequestChangePage Cette fonction définit la procédure à suivre pour modifier les valeurs envoyées au serveur de telle façon que la sélection change de page.
		 *         Cette fonction doit présenter la signature suivante :
		 *         function inUploadRequestChangePage(inValues:Object, inDirection:int):Object
		 *         o inValues Cet objet représente les critères de sélection envoyés au serveur.
		 *                    Cet objet est fourni au présent composant via la méthode setValues().
		 *         o inDirection Pour avancer d'une page : +1.
		 *                       Pour reculer d'une page : -1.
		 *  @param inColumns Ce tableau contient la liste des colonnes qui constituent la grille de visualisation.
		 */
		
		public function DataGridBrowser(inServerUrl:String, inDownloadedDataToXml:Function, inDownloadedDataToControl:Function, inUploadRequestChangePage:Function, inColumns:Array):void
		{
			super();
			__downloadedDataToXml     = inDownloadedDataToXml;
			__downloadedDataToControl = inDownloadedDataToControl;
			__uploadRequestChangePage = inUploadRequestChangePage;
			__columns                 = inColumns;
			__serverUrl               = inServerUrl;
			
			__connector = new RemoteObject();
			__connector.endpoint    = __serverUrl;
			__connector.source      = "SortiesCmd";
			__connector.destination = 'zend';
			
			__createInterface();
			__addEventHandlers();
		}

		/**
		 * Cette méthode permet de personnaliser le texte affiché sur le lien qui permet de sélectionner les enregistrements précédents.
		 * @param inValue Label affiché.sur le lien qui permet d'afficher les enregistrements précédents.
		 */
		
		public function set prevLabel(inValue:String):void
		{
			__prevLinkUp.label   = inValue;
			__prevLinkDown.label = inValue;
		}

		/**
		 * Cette méthode permet de personnaliser le texte affiché sur le lien qui permet de sélectionner les enregistrements suivants.
		 * @param inValue Label affiché.sur le lien qui permet d'afficher les enregistrements suivants.
		 */

		public function set nextLabel(inValue:String):void
		{
			__nextLinkUp.label   = inValue;
			__nextLinkDown.label = inValue;
		}
		
		/**
		 * Cette méthode permet de définir une fonction dont le rôle est de générer le texte qui décrit la progression de la pagination.
		 * @param inFormater Fonction qui sera exécutée pour générer le texte qui décrit la progression de la pagination.
		 *        Cette fonction doit présenter la signature suivante :
		 *        function inFormater(inPageNumber:int, inTotalPage:int):String
		 */
		
		public function set paginationLabelFormater(inFormater:Function):void
		{
			__downloadedDataToPagination = inFormater;
		}
		
		/**
		 * Cette méthode permet de définir le texte qui sera affiché si aucun enregistrement n'est sélectionné.
		 * @param inNoDataLabel Texte qui sera affiché si aucun enregistrement n'est sélectionné.
		 */
		
		public function set noDataLabel(inNoDataLabel:String):void
		{
			__noDataLabel = inNoDataLabel;
		}
		
		/**
		 * Cette méthode permet de définir le texte qui sera affiché tant qu'aucune sélection n'a été lancée.
		 * @param inNoSearchLabel Texte qui sera affiché tant qu'aucune sélection n'a été lancée.
		 */
		
		public function set noSearchLabel(inNoSearchLabel:String):void
		{
			__noSearchLabel = inNoSearchLabel;
			__pageRankUp.htmlText   = __noSearchLabel;
			__pageRankDown.htmlText = '';
		}
		
		/**
		 * Cette méthode permet de définir une fonction qui sera exécutée si la requête au serveur se déroule correctement.
		 * @param inOnSucessAction Fonction qui sera exécutée si la requête au serveur se déroule correctement.
		 */
		
		public function set onSucessAction(inOnSucessAction:Function):void
		{
			__onSucessAction = inOnSucessAction;
		}
		
		/**
		 * Cette méthode construit l'interface utilisateur.
		 */
		
		private function __createInterface():void
		{
			var controlsHboxUp:HBox                = new HBox();
			var controlsHboxDown:HBox              = new HBox();
			var controlSpacerUpLeft:Spacer         = new Spacer();
			var controlSpacerDownLeft:Spacer       = new Spacer();
			var controlSpacerUpRight:Spacer        = new Spacer();
			var controlSpacerDownRight:Spacer      = new Spacer();
			var controlToGridSpacerUp:Spacer       = new Spacer();
			var controlToGridSpacerDown:Spacer     = new Spacer();
			
			controlsHboxUp.percentWidth    = 100;
			controlsHboxDown.percentWidth  = 100;

			controlSpacerDownLeft.percentWidth  = 100;
			controlSpacerUpLeft.percentWidth    = 100;
			controlSpacerDownRight.percentWidth = 100;
			controlSpacerUpRight.percentWidth   = 100;
			
			__prevLinkUp   = new LinkButton(); 
			__nextLinkUp   = new LinkButton();
			__prevLinkDown = new LinkButton();
			__nextLinkDown = new LinkButton();
			__pageRankUp   = new Label();
			__pageRankDown = new Label();

			__nextLinkUp.visible   = false;
			__nextLinkDown.visible = false;
			__prevLinkUp.visible   = false;
			__prevLinkDown.visible = false;

			__prevLinkUp.styleName   = 'gridBrowserNavigationControlCSS';
			__nextLinkUp.styleName   = 'gridBrowserNavigationControlCSS';
			__prevLinkDown.styleName = 'gridBrowserNavigationControlCSS';
			__nextLinkDown.styleName = 'gridBrowserNavigationControlCSS';
			__pageRankUp.styleName   = 'gridBrowserRankLabelCSS';
			__pageRankDown.styleName = 'gridBrowserRankLabelCSS';
			
			controlsHboxUp.styleName   = 'gridBrowserFormularCSS';
			controlsHboxDown.styleName = 'gridBrowserFormularCSS';

			this.prevLabel = 'Précédent';
			this.nextLabel = 'Suivant';
			
			controlsHboxUp.addChild(__prevLinkUp);
			controlsHboxUp.addChild(controlSpacerUpLeft);
			controlsHboxUp.addChild(__pageRankUp);
			controlsHboxUp.addChild(controlSpacerUpRight);
			controlsHboxUp.addChild(__nextLinkUp);
			
			controlsHboxDown.addChild(__prevLinkDown);
			controlsHboxDown.addChild(controlSpacerDownLeft);
			controlsHboxDown.addChild(__pageRankDown);
			controlsHboxDown.addChild(controlSpacerDownRight);
			controlsHboxDown.addChild(__nextLinkDown);
			
			controlToGridSpacerUp.height   = 2;
			controlToGridSpacerDown.height = 2;
			
			__dataGrid               = new DataGrid();
			__dataGrid.percentWidth  = 100;
			__dataGrid.percentHeight = 100;
			
			this.addChild(controlsHboxUp);
			this.addChild(controlToGridSpacerUp);
			this.addChild(__dataGrid);
			this.addChild(controlToGridSpacerDown);
			this.addChild(controlsHboxDown);
		}
		
		/**
		 * Cette méthode assigne les gestionnaires d'évènements aux éléments d'interaction.
		 */
		
		private function __addEventHandlers():void
		{
			__prevLinkUp.addEventListener(flash.events.MouseEvent.CLICK,   __previousRequired);
			__nextLinkUp.addEventListener(flash.events.MouseEvent.CLICK,   __nextRequired);
			__prevLinkDown.addEventListener(flash.events.MouseEvent.CLICK, __previousRequired);
			__nextLinkDown.addEventListener(flash.events.MouseEvent.CLICK, __nextRequired);
			__connector.addEventListener(FaultEvent.FAULT,                 __httpFailed);
			__connector.addEventListener(ResultEvent.RESULT,               __httpSuccess);
		}
		
		/**
		 * Cette méthode rend le composant éligible à la destruction.
		 */
		
		public function destroy():void
		{
			__prevLinkUp.removeEventListener(flash.events.MouseEvent.CLICK,   __previousRequired);
			__nextLinkUp.removeEventListener(flash.events.MouseEvent.CLICK,   __nextRequired);
			__prevLinkDown.removeEventListener(flash.events.MouseEvent.CLICK, __previousRequired);
			__nextLinkDown.removeEventListener(flash.events.MouseEvent.CLICK, __nextRequired);
			__connector.removeEventListener(FaultEvent.FAULT,                 __httpFailed);
			__connector.removeEventListener(ResultEvent.RESULT,               __httpSuccess);
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant.
		 *  @remark Cette méthode est censée être appelée lors de l'enregistrement de l'acteur.
		 *          C'est pour cette raison que l'enregistrement des sous composants de sélection a lieu dans cette fonction.
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;
		}
		
		/** Cette méthode permet de définir une fonction, qui sera exécutée par l'acteur, à chaque fois que ce dernier change d'état.
		 *  @param inFunction Fonction exécutée par le composant à chaque fois que ce dernier change d'état.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction == null) { return; }
 			__onModifHd = inFunction;
		}

		/**
		 * Cette méthode permet d'extraire l'enregistrement sélectionné sur la grille de visualisation.
		 * @return La méthode retourne un objet qui contient les valeurs associées à l'objet sélectionné.
		 */
		
		public function getSelectedValues():Object
		{
			return null;
		}
		
		/**
		 * Cette méthode permet de définir le critère de sélection des enregistrements (clause WHERE).
		 * Elle lance le téléchargement des valeurs initiales.
		 * @param inValues Cet objet contient les valeurs nécessaires à la sélection des enregistrements.
		 *        L'interprétation de ces propriétés est faite sur le serveur.
		 *        Cet objet est entièrement construit par l'utilisateur de la présente classe car son contenu est lié au fonctionnel dans lequel s'intègre le présent composant.
		 * @param inDispatchEvt Dans le cas particulier de ce composant, cet argument n'est pas utilisé.
		 * @remark Contrairement aux autres composants de cette librairie, cette méthode ne permet pas d'associer au composant des propriétés.
		 * @warning Le popup, lancé par la méthode, a besoin d'une fenêtre parente... Cette méthode ne peut donc être appelée que lorsque le composant s'est vu assigner un parent.
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			__where = inValues;
			
			// Lancement du chargement des enregistrements.
			__popup = PopUpManager.createPopUp(this.parent, components.popups.WaitBox, true);
			PopUpManager.centerPopUp(__popup);

			// Transmission de la requête au serveur.
			// Le résultat sera obtenu dans le gestionnaire d'évènement exécuté en cas de réponse positive du serveur.			
			__connector.getSorties(inValues);
		}
		
		/**
		 * Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			__prevLinkUp.enabled = true;
			__nextLinkUp.enabled = true;
			__dataGrid.enabled = true;
		}
		
		/**
		 * Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			__prevLinkUp.enabled = false;
			__nextLinkUp.enabled = false;
			__dataGrid.enabled = false;
		}
		
		/**
		 * Cette méthode permet de réinitialiser le composant.
		 * @note Dans le cas particulier de ce composant, cette méthode n'est pas implémentée.
		 */
		
		public function reset(inValues:Object=null):void
		{
		}
		
		/**
		 * Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur presse le lien qui active la sélection des enregistrements précédents.
		 * @param event Évènement envoyé lorsque le lien qui active la sélection des enregistrements précédents est sollicité.
		 */
		
		private function __previousRequired(event:Event):void
		{
			if (! __loaded) { return; }
			setValues(__uploadRequestChangePage(__where, -1));
		}

		/**
		 * Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur presse le lien qui active la sélection des enregistrements suivants.
		 * @param event Évènement envoyé lorsque le lien qui active la sélection des enregistrements suivants est sollicité.
		 */ 

		private function __nextRequired(event:Event):void
		{
			if (! __loaded) { return; }
			setValues(__uploadRequestChangePage(__where, +1));
		}

		/**
		 * Ce gestionnaire d'évènement est exécuté lorsqu'une erreur est remontée par le lien HTTP avec le serveur.
		 * @param Évènement envoyé lorsque le lien HTTP avec le serveur signale une erreur.
		 */

		private function __httpFailed(event:Event):void
		{
			var hd:IFlexDisplayObject;

			PopUpManager.removePopUp(__popup);
			
			components.popups.PopupMessage.setImage(values.Configuration.koSymbol);
			components.popups.PopupMessage.setMessage("<b>Une erreur s'est produite lors du traitement de votre demande.</b>");
			hd = PopUpManager.createPopUp(this.parent, components.popups.Message, true);
			PopUpManager.centerPopUp(hd);
		}
		
		/**
		 * Ce gestionnaire d'évènement est exécuté lorsque le lien HTTP avec le serveur reçoit une réponse valide du serveur.
		 * @param Évènement envoyé lorsque le lien HTTP avec le serveur reçoit une réponse valide du serveur.
		 */
		
		private function __httpSuccess(result:ResultEvent):void
		{
			var res:Object = result.result;
			var hd:IFlexDisplayObject;
			var pagination:Array;
			
			PopUpManager.removePopUp(__popup);

			try
			{			
				if (! (res.hasOwnProperty('status') && res.hasOwnProperty('method') && res.hasOwnProperty('data'))) { throw new Error(); }
				if (res.status != 1) { throw new Error(); }
			
				// Parfait! Nous avons obtenu la liste des sorties!
				
				// Conversion des valeurs en XML, en utilisant la fonction fournie.
				__provider = __downloadedDataToXml(res.data);
				
				// Extraction des données qui décrivent la progression de la pagination.
				pagination = __downloadedDataToControl(res);
				
				// Remplissage de la grille de visualisation.
				__dataGrid.dataProvider = __provider;
				__dataGrid.columns      = __columns;
				
				// Mise en forme des barres de contrôles.
				__currentPage           = pagination['currentPage'];
				__totalPage             = pagination['totalPages'];
				
				if (__totalPage == 0)
				{
					__pageRankUp.htmlText   = __noDataLabel;
					__pageRankDown.htmlText = '';
				}
				else
				{
					__pageRankUp.htmlText   = __downloadedDataToPagination(__currentPage, __totalPage);
					__pageRankDown.htmlText = __downloadedDataToPagination(__currentPage, __totalPage);
				}
				
				// Désactivation éventuelle des contrôles de navigation.
				if ((__currentPage >= __totalPage) || (__totalPage == 0))
				{ __nextLinkUp.visible = false; __nextLinkDown.visible = false; }
				else
				{ __nextLinkUp.visible = true;  __nextLinkDown.visible = true; }
				
				if ((__currentPage <= 1) || (__totalPage == 0))
				{ __prevLinkUp.visible = false; __prevLinkDown.visible = false; }
				else
				{ __prevLinkUp.visible = true;  __prevLinkDown.visible = true; }
				
				__loaded = true;
				__onSucessAction(res);
			}
			catch (e:Error)
			{
				trace(e.message);
				components.popups.PopupMessage.setImage(values.Configuration.koSymbol);
				components.popups.PopupMessage.setMessage("<b>Une erreur s'est produite lors du traitement de votre demande.</b>");
				hd = PopUpManager.createPopUp(this.parent, components.popups.Message, true);
				PopUpManager.centerPopUp(hd);
			}
		}
		
		/**
		 * Cette méthode définit la mise en page par défaut de la zone qui indique la progression de la pagination.
		 * @param inPageNumber Le numéro de la page courante.
		 * @param inTotalPage Le nombre total de pages.
		 * @return La méthode renvoie une chaîne de caractères qui représente le texte à afficher.
		 */
		
		private function __defaultPageRank(inPageNumber:int, inTotalPage:int):String
		{
			return '<b>' + inPageNumber.toString() + '/' + inTotalPage.toString() + '</b>';
		}
		
	}
}