package components.compositComponents
{
	import components.baseComponents.ExtendedDateField;
	
	import evenements.Action;
	
	import flash.events.Event;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.containers.HBox;
	import mx.containers.VBox;
	import mx.controls.CheckBox;
	import mx.controls.Label;
	import mx.controls.Spacer;
	import mx.events.CalendarLayoutChangeEvent;
	
	import tools.FunctionUtilities;

	/**
	 * Cette classe permet de sélectionner un interval de date.
	 * L'interval est défini par les propriétés suivantes :
	 * o La date de début.
	 * o La durée incompressible de l'interval.
	 * 
	 * Ce composant peut lancer les "Actions" suivants :
	 * o DataSetted
	 * o ValueChanged
	 */

	public class DateRangePicker extends HBox implements IGhostInterface, ICompositeActorInterface, IActorInterface, ISelectorInterface
	{
		private var __setFromDate:CheckBox;
		private var __setToDate:CheckBox;
		private var __fromDate:ExtendedDateField;
		private var __toDate:ExtendedDateField;
		private var __onModifHd:Function = tools.FunctionUtilities.doNothing;
		private var __myName:String  = null;
		private var __fromRange:Date = null;
		private var __deltaRange:int = (60 * 60 * 1000) * 24;
		private var __labelFrom:String;
		private var __labelTo:String;
		
		
		
		/**
		 * Constructeur.
		 * @param inLabelFrom Titre à afficher à droite de la case à cocher qui permet de sélectionner la date de début.
		 *        Ce titre peut être formaté en HTML.
		 * @param inLabelTo Titre à afficher à droite de la case à cocher qui permet de sélectionner la date de fin.
		 *        Ce titre peut être formaté en HTML.
		 */
		
		public function DateRangePicker(inLabelFrom:String, inLabelTo:String)
		{
			super();
			__labelFrom = inLabelFrom;
			__labelTo   = inLabelTo;
			__createInterface();
			__addEventHandlers();
			
		}
		
		/**
		 * Cette méthode assigne les gestionnaires d'évènements à l'interface.
		 */
		
		private function __addEventHandlers():void
		{
			__setFromDate.addEventListener(flash.events.Event.CHANGE,                __fromDateChangeState);
			__setToDate.addEventListener(flash.events.Event.CHANGE,                  __toDateChangeState);
			__fromDate.addEventListener(mx.events.CalendarLayoutChangeEvent.CHANGE,  __fromDateChanged);
			__toDate.addEventListener(mx.events.CalendarLayoutChangeEvent.CHANGE,    __toDateChanged);
		}
		
		/**
		 * Cette méthode rend le composant éligible à la destruction.
		 */
		
		public function destroy():void
		{
			__setFromDate.removeEventListener(flash.events.Event.CHANGE,                __fromDateChangeState);
			__setToDate.removeEventListener(flash.events.Event.CHANGE,                  __toDateChangeState);
			__fromDate.removeEventListener(mx.events.CalendarLayoutChangeEvent.CHANGE,  __fromDateChanged);
			__toDate.removeEventListener(mx.events.CalendarLayoutChangeEvent.CHANGE,    __toDateChanged);
		}
		
		/**
		 * Cette méthode crée l'interface utilisateur.
		 */
		
		private function __createInterface():void
		{
			var checkVbox:VBox     = new VBox();
			var spacerLeft:Spacer  = new Spacer();
			var labelVbox:VBox     = new VBox();
			var spacerRight:Spacer = new Spacer();
			var dateVbox:VBox      = new VBox();
			var labelFrom:Label    = new Label();
			var labelTo:Label      = new Label();
			
			__setFromDate      = new CheckBox();
			__setToDate        = new CheckBox();
			__fromDate         = new ExtendedDateField();
			__toDate           = new ExtendedDateField();
			labelFrom.htmlText = __labelFrom;
			labelTo.htmlText   = __labelTo;
			
			checkVbox.addChild(__setFromDate);
			checkVbox.addChild(__setToDate);
			
			labelVbox.addChild(labelFrom);
			labelVbox.addChild(labelTo);
			
			dateVbox.addChild(__fromDate);
			dateVbox.addChild(__toDate);
			
			__fromDate.enabled  = false;
			__toDate.enabled    = false;
			__setToDate.enabled = false;
			
			this.addChild(checkVbox);
			this.addChild(spacerLeft);
			this.addChild(labelVbox);
			this.addChild(spacerRight);
			this.addChild(dateVbox);
		}
		
		/** Cette méthode permet de définir une fonction, qui sera exécutée par l'acteur, à chaque fois que ce dernier change d'état.
		 *  @param inFunction Fonction exécutée par le composant à chaque fois que ce dernier change d'état.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction == null) { return; }
 			__onModifHd = inFunction;
		}
				
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant.
		 *  @remark Cette méthode est censée être appelée lors de l'enregistrement de l'acteur.
		 *          C'est pour cette raison que l'enregistrement des sous composants de sélection a lieu dans cette fonction.
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;
		}
		
		/**
		 * Cette méthode renvoie les dates de début et de fin de l'interval sélectionné.
		 * @return La méthode renvoie un objet qui contient les propriétés suivantes :
		 *         o 'from': Un objet de type Date qui représente la date de début de l'interval.
		 *            Cette propriété peut éventuellement prendre la valeur null (si aucune date n'a été sélectionnée).
		 *         o 'to': Un objet de type Date qui représente la date de fin de l'interval.
		 *            Cette propriété peut éventuellement prendre la valeur null (si aucune date n'a été sélectionnée).
		 */
		
		public function getSelectedValues():Object
		{
			return { 'from':__fromDate.selectedDate, 'to':__toDate.selectedDate };
		}

		/**
		 * Cette méthode permet d'initialiser les limites de la sélection.
		 * @param inValues Valeurs d'initialisation. Cet objet présente les propriétés suivantes :
		 *        - from: Date de début.
		 *                Si cette propriété n'est pas renseignée, le composant ne présente pas de limite temporelle de début.
		 *        - delta: Interval minimum entre la date de fin et la data de début. Par défaut, l'interval est 24h.
		 *                 Cette valeur est exprimée en ms.
		 * @param inDispatchEvt Cet argument permet de spécifier si le composant doit envoyer un évènement suite à la mise à jour du lieu.
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			var now:Date = new Date();
			
			if (inValues.hasOwnProperty('from'))  { __fromRange  = inValues.from;  }
			if (inValues.hasOwnProperty('delta')) { __deltaRange = inValues.delta; }
			if (__fromRange != null) { __fromDate.enableRangeStart(__fromRange); }
			
			__onModifHd(getSelectedValues());
			if (inDispatchEvt) { dispatchEvent(new Action('Action', __myName, "DataSetted", true)); }
		}

		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			__setFromDate.enabled  = true;
			__setFromDate.selected = false; __changeStateFromDate();
		}

		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			__setFromDate.enabled  = false;
			__setFromDate.selected = false; __changeStateFromDate();
			__onModifHd(getSelectedValues());
		}

		/** Cette méthode réinitialise le composant.
		 *  Dans le cas de ce composant, cette méthode n'est pas implémentée.
		 */
		
		public function reset(inValues:Object=null):void { }
		
		/**
		 * Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur active la sélection de la date de début des sorties.
		 * @param event Évènement envoyé lorsque l'utilisateur clique sur la case à cocher.
		 */
		
		private function __fromDateChangeState(event:Event):void
		{
			__changeStateFromDate();
			__onModifHd(getSelectedValues());
			dispatchEvent(new Action('Action', __myName, "ValueChanged", true));
		}

		/**
		 * Cette méthode implémente les actions à effectuer lorsque le calendrier qui permet de sélectionner la date de début de l'interval change d'état.
		 */

		private function __changeStateFromDate():void
		{
			var now:Date = new Date();
			
			if (__setFromDate.selected)	{
											if (__fromRange != null) 	{
																			__fromDate.enableRangeStart(__fromRange);
																			__fromDate.selectedDate = __fromRange;
																		}
											else { __fromDate.selectedDate = now; }
											__fromDate.enabled      = true;
											__setToDate.enabled     = true;

										}
			else	{
						// On désactive entièrement la sélection de la date de début.
						// Et on annule la sélection éventuelle de la date de début.
						__fromDate.selectedDate = null;
						__fromDate.enabled      = false;
						
						// On désactive entièrement la sélection de la date de fin.
						// Et on annule la sélection éventuelle de la date de fin.
						__toDate.selectedDate   = null;
						__toDate.enabled        = false; // L'utilisateur ne peut plus sélectionner une date de fin.
						__setToDate.enabled     = false; // L'utilisateur ne peut plus cliquer sur la case à cocher.
						__setToDate.selected    = false; // La case n'est plus cochée.
					}
					
			
		}

		/**
		 * Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur active la sélection de la date de fin de l'interval.
		 * @param event Évènement envoyé lorsque l'utilisateur clique sur la case à cocher.
		 */
		
		private function __toDateChangeState(event:Event):void
		{
			__changeStateToDate();
			__onModifHd(getSelectedValues());
			dispatchEvent(new Action('Action', __myName, "ValueChanged", true));
		}
		
		/**
		 * Cette méthode implémente les actions à effectuer lorsque le calendrier qui permet de sélectionner la date de fin de l'interval change d'état.
		 */

		private function __changeStateToDate():void
		{
			// On calcule la date utilisée pour initialiser le sélecteur de fin d'interval.
			var start:Date = new Date(__fromDate.selectedDate.getTime() + __deltaRange);
			
			if (__setToDate.selected)	{
											__toDate.enabled      = true;
											__toDate.selectedDate = start;
											__toDate.enableRangeStart(start);
										}
			else	{
						__toDate.selectedDate = null;
						__toDate.enabled      = false;
					}
		}
		
		/**
		 * Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur modifie la date de début.
		 * @param event Évènement envoyé lorsque l'utilisateur modifie la date de début.
		 */
		
		private function __fromDateChanged(event:Event):void
		{
			if (__toDate.selectedDate != null)
			{
				if (__toDate.selectedDate.getTime() - __fromDate.selectedDate.getTime() < __deltaRange)
				{ __toDate.selectedDate = new Date(__fromDate.selectedDate.getTime() + __deltaRange); }
			}
			dispatchEvent(new Action('Action', __myName, "ValueChanged", true));
			__onModifHd(getSelectedValues());
		}
		
		/**
		 * Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur modifie la date de fin.
		 * @param event Évènement envoyé lorsque l'utilisateur modifie la date de fin.
		 */
		
		private function __toDateChanged(event:Event):void
		{
			dispatchEvent(new Action('Action', __myName, "ValueChanged", true));
			__onModifHd(getSelectedValues());
		}
		
	}
}