package components.compositComponents
{
	import components.baseComponents.HttpComboBox;
	
	import evenements.Action;
	
	import flash.events.Event;
	
	import gestionnaires.Actor;
	import gestionnaires.EventHandlerControler;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.containers.VBox;
	import mx.controls.ComboBox;
	import mx.controls.Label;
	import mx.events.ListEvent;
	
	import tools.DataProviderTools;
	import tools.FunctionUtilities;
	
	import values.GeoData;

	/** Cette classe implémente un composant qui permet de sélectionner un lieu, composé des éléments suivants :
	 *  o La région.
	 *  o Le département.
	 *  o La localité.
	 * 
	 *  @warning Ce composant est associé aux variables de configurations définies dans le fichier AS "values.GeoData".
	 *           Il est aussi associé aux fichiers XML de configuration, présents sur le serveur, et qui contiennent les listes de codes postaux pour chaque département.
	 * 
	 *  Ce composant peut diffuser les actions suivantes :
	 *  o DataSetted - Cela signifie que l'action d'initialisation du composant a été effectuée.
	 *                 Note: Cet évènement ne signifie pas que toutes les valeurs (région, département, code postal) ont été initialisées.
	 *                 Cet évènement signifie que seules les valeurs concernées par la mise à jour ont été mise à jour.
	 *                 Si l'initialisation ne concerne que la région, alors cela signifie que seule la région a été mise à jour.
	 *  o RegionChanged - Attention : Cet évènement signifie que l'état du composant de sélection a changé. Cela ne signifie pas nécessairement que la valeur de la région a changé.
	 *  o DepartementChanged - Attention : Cet évènement signifie que l'état du composant de sélection a changé. Cela ne signifie pas nécessairement que la valeur du dépatement a changé.
	 *  o ZipChanged - Attention : Cet évènement signifie que l'état du composant de sélection a changé. Cela ne signifie pas nécessairement que la valeur du code postal a changé.
	 */

	public class GeoPicker extends VBox implements IGhostInterface, ISelectorInterface, IActorInterface, ICompositeActorInterface
	{
		private var __evtHd:EventHandlerControler = new EventHandlerControler();
		private var __regionCombo:ComboBox        = new ComboBox();
		private var __departementCombo:ComboBox   = new ComboBox();
		private var __zipCombo:HttpComboBox       = new HttpComboBox();
		private var __actionHandler:Actor         = new Actor();
		private var __myName:String               = null;
		private var __localityUrl:String          = null;
		private var __preselectedZip:String       = null;
		private var __onModifHd:Function          = tools.FunctionUtilities.doNothing;
		private var __dispatchEvent:Boolean       = false;
		private var __settingAction:Boolean       = false;
		private var __labels:Object               = null;
		private var __css:Object                  = null;
		private	var __labelRegion:Label           = null;
		private	var __labelDepartement:Label      = null;
		private	var __labelZip:Label              = null;
		
		/** Crée un composant pour sélectionner un lieu, composé de 3 informations : La région, le département, le code postal.
		 *  @param inLocalityUrl URL vers le répertoire qui contient les fichiers XML représentant les listes de codes postaux.
		 *  @param inLabels Cet objet définit les éventuels titres affichés au dessus des menus de sélection.
		 *         Les propriétés sont:
		 *         o 'region': titre au dessus du menu de sélection de la région.
		 *         o 'departement': titre au dessus du menu de sélection du département.
		 *         o 'zip': : titre au dessus du menu de sélection du code postal.
		 *         Il est possible de donner à ce paramètre la valeur null.
		 *  @param inCSS Cet objet contient les noms des classes CSS à appliquer.
		 *         Les propriétés sont les suivantes :
		 *         o 'geoPickerLabel': Nom de la classe CSS qui détermine l'apparence des titres.
		 *         o 'geoPickerCombo': Nom de la classe CSS qui détermine l'apparence des listes de sélection déroulantes.
		 *         o 'geoPickerContainer': Nom de la classe CSS qui détermine l'apparence du conteneur.
		 */
		
		public function GeoPicker(inLocalityUrl:String, inLabels:Object=null, inCSS:Object=null)
		{
			super();
			__localityUrl = inLocalityUrl;
			__labels      = inLabels;
			__css         = inCSS;
			__createGeoPicker();
			__setEventHandler();
		}
		
		/** Cette méthode permet de définir une fonction, qui sera exécutée par l'acteur, à chaque fois que ce dernier change d'état.
		 *  @param inFunction Fonction exécutée par le composant à chaque fois que ce dernier change d'état.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction == null) { return; }
 			__onModifHd = inFunction;
		}
		
		/** Cette méthode est utilisée pour déclarer les évènements traités par cette classe.
		 */
		 		
		private function __setEventHandler():void
		{
			__evtHd.addEventHandler(__regionCombo,      ListEvent.CHANGE,                 __regionChanged);
			__evtHd.addEventHandler(__departementCombo, ListEvent.CHANGE,                 __departementChanged);
			__evtHd.addEventHandler(__zipCombo,         'Action',                         __gotAction);
		}

		/** Ce gestionnaire d'évènement est exécuté lorsqu'une action est reçue. 
		 * 	@param event Action envoyée.
		 */

		private function __gotAction(event:Action):void
		{
			if (__zipChanged(event))
			{
				__onModifHd(getSelectedValues());
				dispatchEvent(new Action('Action', __myName, "ZipChanged", true));
			}
		}
		
		/** Cette fonction détermine si une action correspond à un changement de valeur du code postal.
		 *  @param event Action envoyée.
		 */
		
		private function __zipChanged(event:Action):Boolean
		{
			if (event.target      != __zipCombo)     { return false; }
			if (event.getAction() != 'DataSelected') { return false; }
			return true;
		}
		
		/** Ce gestionnaire d'évènement est exécuté lorsque la boîte de sélection de la région change.
		 *  @param event Évènement envoyé par la boîte de sélection.
		 */
		
		private function __regionChanged(event:Event):void
		{
			// On réinitialise toutes les données liées à la région.
			__departementCombo.dataProvider = null
			__departementCombo.enabled      = false;
			__zipCombo.dataProvider         = null;
			__zipCombo.enabled              = false;

			__setDepartementItem(__regionCombo.selectedItem);
			dispatchEvent(new Action('Action', __myName, "RegionChanged", true));
			__onModifHd(getSelectedValues());
		}

		/** Cette méthode assigne une présélection au composant de sélection du département.
		 *  @param inSelectedRegionItem Item qui représente la région sélectionnée.
		 */

		private function __setDepartementItem(inSelectedRegionItem:Object):void
		{
			// Si l'élément sélectionné est le premier de la liste ("---"), alors on ne fait rien.
			// Il faut annuler les sélections du département et du code postal.
			if (inSelectedRegionItem.@data == "00") { return; }
			
			// On extrait la liste des départements associés à la région sélectionnée.
			// Puis on l'associe au composant de sélection.	
			__departementCombo.dataProvider = values.GeoData.regions.region.(@data == inSelectedRegionItem.@data).departement;
			__departementCombo.enabled = true;
		}

		/** Ce gestionnaire d'évènement est exécuté lorsque la boîte de sélection du département change d'état.
		 *  @param event Évènement envoyé par la boîte de sélection.
		 */
		 
		private function __departementChanged(event:Event):void
		{
			var values:Object;
			var departementItem:Object = __departementCombo.selectedItem;
			
			
			// if (departementItem == null) { trace ('departementItem is null'); return; }
			
			// On réinitialise toutes les données liées au département.
			__zipCombo.dataProvider    = null;
			__zipCombo.disable();
			
			// Si l'élément sélectionné est le premier de la liste ("---"), alors on ne fait rien.
			// Il faut annuler la sélection de la localité.
			if (departementItem.@data == "00")
			{
				if		(__dispatchEvent && __settingAction) { dispatchEvent(new Action('Action', __myName, "DataSetted", true)); __dispatchEvent = false; __settingAction = false; }
				else	{ dispatchEvent(new Action('Action', __myName, "DepartementChanged", true)); }
				__onModifHd(getSelectedValues());
				return;
			}
			
			// On charge la liste des localités associées au département sélectionné.
			values = { 	url:__localityUrl + '/geodata-' + __regionCombo.selectedItem.@data + '-' + __departementCombo.selectedItem.@data + '-zips.xml',
						prePlugin:__localityLoaderPrePlugin,
						postPlugin:__localityLoaderPostPlugin,
						format:'text',
						method:'GET',
						values:null,
						item:'zip',
						label:'label',
						data:'data'};
			if (__preselectedZip != null) { values['selectedData'] = __preselectedZip; __preselectedZip = null; }
			
			// La ligne ci-dessous va lancer le téléchargement de la liste des codes postaux.
			// La réussite du téléchargement va déclencher l'exécution du greffon "__localityLoaderPostPlugin()".
			// La fonction __onModifHd() sera exécutée dans le greffon "__localityLoaderPostPlugin()".
			// trace ('lancement du téléchargement du ZIP!');
			__zipCombo.setValues (values, false);
			
			// Note: L'action sera envoyée lors de l'injection de la liste de codes postaux téléchargée dans le composant de sélection.
			// CF Cette association est effectuée dans le greffon "__localityLoaderPostPlugin()".
		}

		/** Cette fonction est utilisée par le composant responsable du chargement des codes postaux, via HTTP.
		 *  Son rôle est d'effectuer des actions suite à l'initialisation du composant de type "ComboBox".
		 *  @param value Cette variable contiendra le document téléchargé.
		 *  @return La méthode renvoie un document compatible avec l'utilisation de la ComboBox.
		 */

		private function __localityLoaderPostPlugin(value:Object):Object
		{
			// Si le chargement se passe bien, alors on active la boîte de sélection.
			__zipCombo.enable();
			if (__dispatchEvent && __settingAction) { dispatchEvent(new Action('Action', __myName, "DataSetted", true)); __dispatchEvent = false; __settingAction = false; }
			else { dispatchEvent(new Action('Action', __myName, "DepartementChanged", true)); }
			__onModifHd(getSelectedValues());
			return null;
		}

		/** Cette fonction est utilisée par le composant responsable du chargement des codes postaux, via HTTP.
		 *  Son rôle est de convertir le document téléchargé de façon à le rendre compatible avec l'utilisation de la boîte de sélection du type "ComboBox".
		 *  @param value Cette variable contiendra le document téléchargé.
		 *  @return La méthode renvoie un document compatible avec l'utilisation de la ComboBox.
		 */
		
		private function __localityLoaderPrePlugin(value:Object):Object
		{
			return XML(value);
		}
		
		/** Cette méthode crée l'interface du composant.
		 */
		
		private function __createGeoPicker():void
		{

			
			// Fabrication des listes de sélection déroulantes.
			__regionCombo.dataProvider = values.GeoData.regions.region;
			__regionCombo.enabled      = true;
			__regionCombo.labelField   = "@label";
			__regionCombo.width        = 330;
			
			__departementCombo.dataProvider = null;
			__departementCombo.enabled      = false;
			__departementCombo.labelField   = '@label';
			__departementCombo.width        = 330;
			
			__zipCombo.dataProvider  = null;
			__zipCombo.enabled       = false;
			__zipCombo.labelField    = '@label';
			__zipCombo.width         = 330;
			
			// Définition des labels.
			if (__labels != null)
			{
				__labelRegion      = new Label();
				__labelDepartement = new Label();
				__labelZip         = new Label();
				
				__labelRegion.text      = __labels.region;
				__labelDepartement.text = __labels.departement;
				__labelZip.text         = __labels.zip;
			}

			// Application des CSS.
			__setCss();
		
			// Injection des composants unitaires dans le conteneur.
			if (__labels != null) { this.addChild(__labelRegion); }
			this.addChild(__regionCombo);
			if (__labels != null) { this.addChild(__labelDepartement); }
			this.addChild(__departementCombo);
			if (__labels != null) { this.addChild(__labelZip); }
			this.addChild(__zipCombo);
		}
		
		/**
		 * Cette méthode applique le CSS.
		 */
		
		private function __setCss():void
		{
			if (__css != null)
			{
				if (__css.hasOwnProperty('geoPickerLabel'))
				{				
					__labelRegion.styleName      = __css.geoPickerLabel;
					__labelDepartement.styleName = __css.geoPickerLabel;
					__labelZip.styleName         = __css.geoPickerLabel;
				}
				
				if (__css.hasOwnProperty('geoPickerCombo'))
				{
					__regionCombo.styleName      = __css.geoPickerCombo;
					__departementCombo.styleName = __css.geoPickerCombo;
					__zipCombo.styleName         = __css.geoPickerCombo;
				}
				
				if (__css.hasOwnProperty('geoPickerContainer')) { this.styleName = __css.geoPickerContainer;  }
			}
		}
		
		
		/** Cette méthode libère les ressources allouées au composant.
		 */		
		
		public function destroy():void
		{
			__zipCombo.destroy();
			__evtHd.removeAllEventHandlers();
		}

		/** Cette méthode renvoie les valeurs sélectionnées par l'utilisateur.
		 *  @return La méthode renvoie un objet qui contient la description du lieu sélectionné.
		 *          L'objet retourné contient les propriétés suivantes :
		 *          o region (La valeur de cette propriété peut valoir null)
		 *          o departement (La valeur de cette propriété peut valoir null)
		 *          o zip (La valeur de cette propriété peut valoir null)
		 *  @warning Lorsque le composant est initialisé de façon automatique (sans intervention humaine), il peut y avoir des problèmes.
		 */
	
		public function getSelectedValues():Object
		{
			var values:Object = new Object();
			var zip:Object;
			
			values['zip'] = null;
			zip = __zipCombo.getSelectedValues();
			
			if (__regionCombo.selectedItem != null) { values['region'] = __regionCombo.selectedItem.attribute('data'); }
			else	{
						values['region']		= null;
						values['departement']	= null;
						values['zip']			= null;
						return values;
					}
			
			if (__departementCombo.selectedItem != null) { values['departement'] = __departementCombo.selectedItem.attribute('data'); }
			else 	{
						values['departement']	= null;
						values['zip'] 			= null;
						return values;
					}
			
			values['departement'] = __departementCombo.selectedItem.attribute('data');
			if (zip.value != null) 	{
										trace ('ZIP = ' + zip['value']);
										values['zip'] = zip['value'];
									}

			return values;
		}

		/** Cette méthode permet d'initialiser les valeurs des interfaces de sélection.
		 *  @param inValues Cet objet contient l'ensemble des valeurs d'initialisation.
		 *         Les propriétés de cet objet sont :
		 *         o region
		 *         o departement
		 *         o zip
		 *  @param inDispatchEvt Cet argument permet de spécifier si le composant doit envoyer un évènement suite à la mise à jour du lieu.
		 *  @warning L'exécution de cette méthode doit se faire séquentiellement!
		 *           Il ne faut donc pas assigner l'évènement "VALUE_COMMIT".
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			__dispatchEvent = inDispatchEvt;
			
			if (inValues.hasOwnProperty('region'))
			{
				__regionCombo.selectedIndex = tools.DataProviderTools.getComboBoxDataProviderIndex('data', inValues.region, __regionCombo.dataProvider);
				__setDepartementItem(__regionCombo.selectedItem);
			} else	{
						// Si la région n'est pas spécifiée, on ne peut rien faire...
						return;
					}
			
			if (inValues.hasOwnProperty('departement'))
			{
				__departementCombo.selectedIndex = tools.DataProviderTools.getComboBoxDataProviderIndex('data', inValues.departement, __departementCombo.dataProvider);
				
				// Le lancement de l'évènement (ci-après) provoquera le téléchargement de la liste des codes postaux.
				if (inValues.hasOwnProperty('zip'))
				{
					// Attention : L'action qui signale que les données ont été assignées au composant doit être envoyée une fois que la liste des codes postaux a été téléchargée.
					// L'indicateur "__settingAction" permet de différencier, à l'intérieur du greffon de "post téléchargement", s'il s'agit d'une mise à jour programmée, ou d'une mise à jour suite à une action de l'utilisateur.
					__preselectedZip = inValues.zip;
					__settingAction  = true;
					__departementCombo.dispatchEvent(new Event(ListEvent.CHANGE));
					return;
				}
			}
			
			if (__dispatchEvent) { dispatchEvent(new Action('Action', __myName, "DataSetted", true)); __dispatchEvent = false; __settingAction = false; }
		}

		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			__zipCombo.enable();
			__departementCombo.enabled = true;
			__regionCombo.enabled      = true;
		}

		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			__zipCombo.disable();
			__departementCombo.enabled = false;
			__regionCombo.enabled      = false;
		}

		/** Cette méthode réinitialise le composant.
		 */
		
		public function reset(inValues:Object=null):void
		{
			__zipCombo.reset();
			__departementCombo.dataProvider = null;
			__regionCombo.dataProvider      = null;
			__onModifHd(getSelectedValues());
		}

		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant.
		 *  @remark Cette méthode est censée être appelée lors de l'enregistrement de l'acteur.
		 *          C'est pour cette raison que l'enregistrement des sous composants de sélection a lieu dans cette fonction.
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;
			__zipCombo.setName(__myName + '-zip');
			// __actionHandler.register(__myName + '-Locality', __zipCombo);
		}
	}
}