package components.compositComponents
{
	
	
	import components.UIComponents.DateTimePicker;
	
	import evenements.Action;
	
	import gestionnaires.Actor;
	import gestionnaires.EventHandlerControler;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.containers.HBox;
	import mx.containers.VBox;
	import mx.controls.Spacer;
	import mx.controls.Text;
	
	import tools.FunctionUtilities;
	
	/** Cette classe implémente un composant de sélection des informations temporelles relatives à l'organisation d'un évènement.
	 *  Le composant comporte trois sélecteurs :
	 *     o Un sélecteur pour la date de l'évènement.
	 *     o Un sélecteur pour la date limite d'inscription.
	 *     o Un sélecteur pour la date limite de résiliation de l'inscription.
	 *  Le composant gère des contraintes temporelles entre les trois sélecteurs précités.
	 * 
	 *  Ce composant peut diffuser les actions suivantes :
	 *  o DataSetted - Cela signifie que l'action d'initialisation du composant a été effectuée.
	 *                 Note: Cet évènement ne signifie pas que toutes les valeurs (les trois dates) ont été initialisées.
	 *                 Cet évènement signifie que seules les valeurs concernées par la mise à jour ont été mise à jour.
	 *  o PartyDateChanged
	 *  o PartyTimeChanged
	 *  o RegistrationDateChanged
	 *  o RegistrationTimeChanged
	 *  o CancellationTimeChanged
	 *  o CancellationDateChanged
	 */
	
	[ Event( name="Action", type="evenements.Action") ]
	public class PartyTimePicker extends VBox implements IGhostInterface, ISelectorInterface, IActorInterface, ICompositeActorInterface
	{
		private var __myName:String;
		private var __actionHandler:Actor                        = null;
		private var __datePickerDate:DateTimePicker              = null;
		private var __datePickerRegistrationLimit:DateTimePicker = null;
		private var __datePickerCancellationLimit:DateTimePicker = null;
		private var __partyLabel:Text                            = null;
		private var __registrationLabel:Text                     = null;
		private var __cancellationLabel:Text                     = null;
		private var __css:Object                                 = null;
		private var __onModifHd:Function                         = tools.FunctionUtilities.doNothing;
		private var __evtHd:EventHandlerControler                = new EventHandlerControler();
		
		/** Créer un sélecteur des informations temporelles relatives à l'organisation d'un évènement.
		 *  Les éléments de sélection peuvent éventuellement être décalés, vers la droite, par rapport aux titres.
		 *  @param inPartyLabel Titre affiché au-dessus du sélecteur de la date de l'évènement.
		 *  @param inRegistrationLabel Titre affiché au-dessus du sélecteur de la date limite d'inscription.
		 *  @param inCancellationLabel Titre affiché au-dessus du sélecteur de la date limite de résiliation de l'inscription.
		 *  @param inLeftMargin Largeur, en pixels, du décalogue des éléments de sélection.
		 *  @param inCSS Cet objet contient les noms des classes CSS à appliquer.
		 *         Les propriétés sont les suivantes :
		 *         o 'partyTimePickerLabel':     Nom de la classe CSS qui détermine l'apparence des titres.
		 *         o 'partyTimePickerContainer': Nom de la classe CSS qui détermine l'apparence du conteneur.
	 	 *         o 'dateClockContainer':       Nom de la classe CSS qui détermine l'apparence du conteneur.
		 *         o 'dateClockCalendar':        Nom de la classe CSS qui détermine l'apparence du calendrier. 
		 *         o 'clockText':                Nom de la classe CSS qui détermine l'apparence du texte de l'horloge.
		 *         o 'clockButton':              Nom de la classe CSS qui détermine l'apparence des boutons de l'horloge.
		 *         o 'clockBox':                 Nom de la classe CSS qui détermine l'apparence du conteneur de l'horloge.
		 */
		
		public function PartyTimePicker(inPartyLabel:String,
						inRegistrationLabel:String,
						inCancellationLabel:String,
						inLeftMargin:Number,
						inCss:Object=null)
		{
			super();
			__css = inCss; trace ("PartyTimePicker => " + __css.partyTimePickerLabel);
			__actionHandler = new Actor();
			
			__datePickerDate              = new DateTimePicker(true,  null, null, "0", 10, 'french', __css);
			__datePickerRegistrationLimit = new DateTimePicker(false, null, null, "0", 10, 'french', __css);
			__datePickerCancellationLimit = new DateTimePicker(false, null, null, "0", 10, 'french', __css);
			
			__partyLabel                  = new Text(); __partyLabel.htmlText        = inPartyLabel;
			__registrationLabel           = new Text(); __registrationLabel.htmlText = inRegistrationLabel;
			__cancellationLabel           = new Text(); __cancellationLabel.htmlText = inCancellationLabel;
			
			__datePickerDate.setValues({begin: new Date()});
			__createPartyTimePicker(inLeftMargin);
			__setCss();
			__setEventHandler();
		}

		/** Cette méthode permet de définir une fonction, qui sera exécutée par l'acteur, à chaque fois que ce dernier change d'état.
		 *  @param inFunction Fonction exécutée par le composant à chaque fois que ce dernier change d'état.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction == null) { return; }
 			__onModifHd = inFunction;
		}
		
		/** Crée l'interface utilisateur associée au composant.
		 *  @param inSelector Référence vers l'objet de sélection de la date et de l'heure.
		 */
		
		private function __createPartyTimePicker(inLeftMargin:Number):void
		{
			this.addChild(__partyLabel);
			this.addChild(__createSelectorLine(inLeftMargin, __datePickerDate));
			this.addChild(__registrationLabel);
			this.addChild(__createSelectorLine(inLeftMargin, __datePickerRegistrationLimit));
			this.addChild(__cancellationLabel);
			this.addChild(__createSelectorLine(inLeftMargin, __datePickerCancellationLimit));
		}
		
		/**
		 * Cette méthode applique le CSS.
		 */
		
		private function __setCss():void
		{
			if (__css != null)
			{
				if (__css.hasOwnProperty('partyTimePickerLabel'))	{
																		__partyLabel.styleName        = __css.partyTimePickerLabel;
																		__registrationLabel.styleName = __css.partyTimePickerLabel;
																		__cancellationLabel.styleName = __css.partyTimePickerLabel;
																	}
				if (__css.hasOwnProperty('partyTimePickerContainer')) { this.styleName = __css.partyTimePickerContainer; }
			}
		}
		
		/** Cette méthode crée la ligne qui contient le sélecteur de date.
		 *  @param inLeftMargin Largeur, en pixels, du décalogue des éléments de sélection.
		 *  @param inSelector Référence vers l'objet de sélection de la date et de l'heure.
		 *  @return La méthode retourne un objet de type HBox.
		 */
		
		private function __createSelectorLine(inLeftMargin:Number, inSelector:DateTimePicker):HBox
		{
			var hbox:HBox     = new HBox();
			var spacer:Spacer = new Spacer();
			
			spacer.width         = inLeftMargin;
			spacer.percentHeight = 100;
			hbox.addChild(spacer);
			hbox.addChild(inSelector);
			return hbox;
		}
		
		/** Cette méthode est utilisée pour déclarer les évènements traités par cette classe.
		 */		

		// [Inspectable(defaultValue="false")]
		private function __setEventHandler():void
		{
			__evtHd.addEventHandler(this, 'Action', __treatAction);
		}
		
		/** Cette méthode libère les ressources allouées au composant.
		 */		
		
		public function destroy():void
		{
			__evtHd.removeAllEventHandlers();
			__datePickerDate.destroy();
			__datePickerRegistrationLimit.destroy();
			__datePickerCancellationLimit.destroy();
		}
		
		/** Cette méthode est utilisée pour gérer les dépendances entre les composants de sélection.
		 *  @param event Cet évènement correspond à un changement d'état fonctionnel d'un des composants de sélection.
		 *  @remark Les changements d'heure (action = 'TimeChanged') ne nous intéressent pas.
		 *          Les changements de date ou d'heure de la limite de résiliation (source = 'PartyCancellation') ne nous intéressent pas.
		 */
		
		private function __treatAction(event:Action):void
		{
			var action:String;
			
			action = event.getAction();
			// trace ('PartyTimePicker: ' + source + ':' + action);
			
			if (__myName == null) { throw new Error("PartyTimePicker has no name."); }
			
			trace ('=> ' + action);
			
			// switch (source)
			switch (event.target)
			{
				// Sélection de la date de la sortie.
				case __datePickerDate:
				{
					switch (action)
					{
						case 'DateChanged':	{
												__partyDateChanged(__datePickerDate.selectedDate()); __onModifHd(getSelectedValues());
												dispatchEvent(new Action('Action', __myName, "PartyDateChanged", true));
											};	break;
						case 'TimeChanged': { dispatchEvent(new Action('Action', __myName, "PartyTimeChanged", true)); }
						default:	{
										// Cette double vérification assure que la date de résiliation est antérieure à la date de la sortie (par transitivité).
										__checkAndAjustRegistrationDate();
										__checkAndAjustCancellationDate();
										__onModifHd(getSelectedValues());
									}
					}
					
				}	break;
				
				// Sélection de la date limite d'inscription.
				case __datePickerRegistrationLimit:
				{
					switch (action)
					{
						// Remarque: Lors d'un changement de la date, il faut vérifier que l'heure est comprise dans les limites fonctionnelles.
						case 'DateChanged':	{
												__registrationDateChnaged(__datePickerRegistrationLimit.selectedDate());
												dispatchEvent(new Action('Action', __myName, "RegistrationDateChanged", true));
											}
						case 'TimeChanged': { dispatchEvent(new Action('Action', __myName, "RegistrationTimeChanged", true)); }
						default:	{
										// La ligne suivante garantit que la date limite d'inscription ne dépasse pas la date de la sortie.
										__checkAndAjustRegistrationDate();
										__checkAndAjustCancellationDate();
										__onModifHd(getSelectedValues());
									}
					}
				} break;
				
				// Sélection de la date limite de résiliation de l'inscription.
				case __datePickerCancellationLimit:
				{
					switch (action)
					{
						case 'TimeChanged': { dispatchEvent(new Action('Action', __myName, "CancellationTimeChanged", true)); } break; 
						case 'DateChanged': { dispatchEvent(new Action('Action', __myName, "CancellationDateChanged", true)); } break; 
					}
					// La ligne suivante garantit que la date limite de résiliation de l'inscription ne dépasse pas la date d'inscription... et donc la date de la sortie.
					__checkAndAjustCancellationDate();
					__onModifHd(getSelectedValues());
				} break;				
			}			
		}
		
		/** Cette méthode est activée lorsque la date de la sortie est modifiée.
		 *  La date limite d'inscription est comprise entre aujourd'hui et la date de la sortie.
		 *  @param inLimit Date de l'évènement.
		 */
		
		private function __partyDateChanged(inLimit:Date):void
		{
			// trace ("Party Date is " + __datePickerDate.selectedDate().toString());
			__datePickerRegistrationLimit.setValues({begin: new Date()});
			__datePickerRegistrationLimit.setValues({end:   inLimit});
			__datePickerRegistrationLimit.enable();
			__datePickerCancellationLimit.disable();
			__datePickerRegistrationLimit.reset({date:true, time:0});
			__datePickerCancellationLimit.reset({date:true, time:0});
		}

		/** Cette méthode est activée lorsque la date limite d'inscription est modifiée.
		 *  La date limite de désinscription est comprise entre aujourd'hui et la date de la sortie.
		 *  @param inLimit Date limite d'inscription.
		 */
		
		private function __registrationDateChnaged(inLimit:Date):void
		{
			trace ("La date d'incription est modifiée.");
			__debugPrintData('__registrationDateChnaged()');
			__datePickerCancellationLimit.setValues({begin: new Date()});
			__datePickerCancellationLimit.setValues({end:   inLimit});
			__datePickerCancellationLimit.enable();
			__datePickerCancellationLimit.reset({date:true, time:0});
		}
		
		/** Cette fonction vérifie, et éventuellement ajuste, la date limite d'inscription, par rapport à la date de la sortie.
		 *  La date limite d'inscription doit, évidemment, se situer avec la date de la sortie, sur la ligne du temps.
		 *  @return Si la date limite d'inscription est valide, la fonction renvoie la valeur "true".
		 *          Sinon, la fonction renvoie la valeur "false". 
		 */
		
		private function __checkAndAjustRegistrationDate():Boolean
		{
			__debugPrintData('__checkAndAjustRegistrationDate()');
			// trace ('Execute __checkAndAjustRegistrationDate()');
			if (__datePickerRegistrationLimit.selectedDateTime() == null) { return true; }
			
			if (__datePickerRegistrationLimit.selectedDateTime().time <= __datePickerDate.selectedDateTime().time) { return true; }
			// Attention, il ne faut pas envoyer d'évènement suite au changement de l'heure! Sinon, on va provoquer un débordement de pile!
			// C'est pour cette raison que le deuxième argument vaut la valeur "false".
			__datePickerRegistrationLimit.setValues({time:__datePickerDate.getTimeAsString()}, false);
			return false;
		}
		
		/** Cette fonction vérifie, et éventuellement ajuste, la date limite de résiliation de l'inscription, par rapport à la date de limite d'inscription.
		 *  La date limite de résiliation de l'inscription doit, évidemment, se situer avec la date de limite d'inscription, sur la ligne du temps.
		 *  @return Si la date limite de résiliation de l'inscription est valide, la fonction renvoie la valeur "true".
		 *          Sinon, la fonction renvoie la valeur "false". 
		 */
		
		private function __checkAndAjustCancellationDate():Boolean
		{
			__debugPrintData('__checkAndAjustCancellationDate()');
			// trace ('Execute __checkAndAjustCancellationDate()');
			// Attention! Dans le cas où cette fonction est exécutée lors d'un changement de date de la sortie, la date limite d'inscription a été réinitialisée.
			if (__datePickerRegistrationLimit.selectedDateTime() == null) { return true; }
			if (__datePickerCancellationLimit.selectedDateTime() == null) { return true; }
			if (__datePickerCancellationLimit.selectedDateTime().time <= __datePickerRegistrationLimit.selectedDateTime().time) { return true; }
			// Attention, il ne faut pas envoyer d'évènement suite au changement de l'heure! Sinon, on va provoquer un débordement de pile!
			// C'est pour cette raison que le deuxième argument vaut la valeur "false".
			__datePickerCancellationLimit.setValues({time:__datePickerRegistrationLimit.getTimeAsString()}, false);
			return false;			
		}

		private function __debugPrintData(st:String=''):void
		{
			trace ('');
			trace (st);
			if (__datePickerDate.selectedDateTime() != null)              { trace ('__checkAndAjustCancellationDate(): Party        ' + __datePickerDate.selectedDateTime().time); }
			if (__datePickerRegistrationLimit.selectedDateTime() != null) { trace ('__checkAndAjustCancellationDate(): Registration ' + __datePickerRegistrationLimit.selectedDateTime().time); }
			if (__datePickerCancellationLimit.selectedDateTime() != null) { trace ('__checkAndAjustCancellationDate(): Cancellation ' + __datePickerCancellationLimit.selectedDateTime().time); }
		}
		
		/** Cette méthode renvoie les valeurs sélectionnées par l'utilisateur.
		 *  L'objet renvoyé se présente sous la forme d'une arborescence.
		 *  Il comporte trois branches principales : une branche pour chaque composant de sélection unitaire.
		 *    o party: La date de l'évènement. Cette propriété peut pointer vers une la valeur null (si aucune date n'a été sélectionnée)..
		 *    o registration: La date de clôture des inscriptions. Cette propriété peut pointer vers une la valeur null (si aucune date n'a été sélectionnée)..
		 *    o cancellation: La date limite pour résilier son inscription. Cette propriété peut pointer vers une la valeur null (si aucune date n'a été sélectionnée)..
		 *  Chacune de ces branches contient les propriétés suivantes :
		 *    o hour    // L'heure.
		 *    o minute  // Le nombre de minutes depuis le début de l'heure.
		 *    o value   // Le nombre total de minutes depuis 0h.
		 *    o date    // 1 à 31
		 *    o day     // 0 pour Sunday, 1 pour Monday
		 *    o year    // L'année
		 *    o month   // 0 pour January, 1 pour February
		 * @return La méthode renvoie un objet qui contient la date et l'heure sélectionnée.
		 */
		
		public function getSelectedValues():Object
		{
			var values:Object = new Object();
			values['party']        = __datePickerDate.getSelectedValues();
			values['registration'] = __datePickerRegistrationLimit.getSelectedValues();
			values['cancellation'] = __datePickerCancellationLimit.getSelectedValues();
			return values;
		}
		
		/** Cette méthode permet d'initialiser les valeurs des interfaces de sélection.
		 *  @param inValues Cet objet contient l'ensemble des valeurs d'initialisation.
		 *         Cet objet se présente sous la forme d'une arborescence comportant 3 branches principales :
		 *           o party: La date de l'évènement.
		 *           o registration: La date de clôture des inscriptions.
		 *           o cancellation: La date limite pour résilier son inscription.
		 *         Chacune de ces branches contient les propriétés suivantes :
		 *           o time: Valeur utilisée pour initialiser l'horloge. Deux formats sont utilisables :
		 *                   - Un nombre entier compris entre 0 et 1439 (inclus), qui représente le nombre de minutes écoulées depuis 0h.
		 *                   - Une chaîne "HH:MM". 
		 *           o step: Valeur de la marche entre deux valeurs successives des minutes.
		 *           o date: Objet de type Date.
		 *           o bagin: Date de bébut de l'interval de sélection.
		 *           o end: Date de fin de l'interval de sélection.
		 *  @param inDispatchEvt Cet argument permet de spécifier si le composant doit envoyer un évènement suite à la mise à jour de l'heure.
		 *  @remark Des contrôles de cohérence sont effectués sur les valeurs de chaque composant de sélection.
		 *          En particulier, les valeurs plages de sélection autorisées sont appliquées.
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
			if (inValues.hasOwnProperty('party'))
			{
				__datePickerDate.setValues(inValues['party']);
				if (inValues['party'].hasOwnProperty('date')) { __partyDateChanged(__datePickerDate.selectedDate()); }
			}

			// Note: L'existence de la date limite d'inscription suppose que la date de l'évènement a été définie.
			if (inValues.hasOwnProperty('registration'))
			{
				if (! inValues['party'].hasOwnProperty('date')) { throw new Error("Limit date for registration implies that party's date exists!"); }
				__datePickerRegistrationLimit.setValues(inValues['registration']);
				if (inValues['registration'].hasOwnProperty('date')) { __registrationDateChnaged(__datePickerRegistrationLimit.selectedDate()) }
				__datePickerRegistrationLimit.enable();
			}

			// Note: L'existence de la date limite de résiliation de l'inscription suppose que la date de l'évènement a été définie.
			if (inValues.hasOwnProperty('cancellation'))
			{
				if (! inValues['registration'].hasOwnProperty('date')) { throw new Error("Limit date for cancellation implies that party's registration date exists!"); }
				__datePickerCancellationLimit.setValues(inValues['cancellation']);
				__datePickerCancellationLimit.enable();
			}
			
			if (inDispatchEvt) { dispatchEvent(new Action('Action', __myName, "DataSetted", true)); }
			__onModifHd(getSelectedValues());
		}
		
		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			__datePickerDate.enable();
			__datePickerRegistrationLimit.enable();
			__datePickerCancellationLimit.enable();
		}
		
		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			__datePickerDate.disable();
			__datePickerRegistrationLimit.disable();
			__datePickerCancellationLimit.disable();
		}
		
		/** Cette méthode réinitialise le composant.
		 */
		
		public function reset(inValues:Object=null):void
		{
			__datePickerDate.reset();
			__datePickerRegistrationLimit.reset();
			__datePickerCancellationLimit.reset();
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant.
		 *  @remark Cette méthode est censée être appelée lors de l'enregistrement de l'acteur.
		 *          C'est pour cette raison que l'enregistrement des sous composants de sélection a lieu dans cette fonction.
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;
			__datePickerDate.setName(__myName);
			__datePickerRegistrationLimit.setName(__myName);
			__datePickerCancellationLimit.setName(__myName);
		}
		
	}
}