package components.compositComponents
{
	import flash.events.Event;
	
	import interfaces.IActorInterface;
	import interfaces.ICompositeActorInterface;
	import interfaces.IGhostInterface;
	import interfaces.ISelectorInterface;
	
	import mx.containers.HBox;
	import mx.controls.Image;
	import mx.controls.Spacer;
	import mx.controls.TextInput;
	import mx.events.FlexEvent;
	
	import tools.FunctionUtilities;

	/** Cette classe implémente un filtre anti-robot.
	 *  @warning Les images envoyées par le serveur au filtre doivent comporter uniquement les caractères suivants :
	 *           Les chiffres de 0 à 9.
	 *           Les 26 lettres de l'alphabet, en minuscule, et non accentuées.
	 */

	public class RobotFilter extends HBox implements IActorInterface, IGhostInterface, ISelectorInterface, ICompositeActorInterface
	{
		private var __image:Image = new Image();
		private var __textInput:TextInput = new TextInput();
		private var __session:String;
		private var __charNumber:Number;
		private var __myName:String;
		private var __onModifHd:Function = tools.FunctionUtilities.doNothing;
		 
		/** Cette méthode construit le filtre.
		 *  @param inImageSenderUrl URL vers le service WEB chargé d'attribuer des images.
		 *  @param inSession Nom de l'image. Ce nom est utilisé par le serveur pour associer l'image à une chaîne de caractères qui représente l'image (sous forme textuelle).
		 *                   Typiquement, cette valeur sera initialisée avec l'identifiant de session.
		 *  @param inSize Nombre de caractères affichés sur l'image.
		 */
		
		public function RobotFilter(inImageSenderUrl:String, inSession:String, inSize:Number)
		{
			super();
			__session    = inSession;
			__charNumber = inSize;
			__image.load(inImageSenderUrl);
			createRobotFilter();
			
		}
		
		/** Cette méthode construit l'interface utilisateur.
		 */
		
		private function createRobotFilter():void
		{
			var spacer:Spacer = new Spacer();

			spacer.width = 50;
			spacer.percentHeight = 100;
			__textInput.maxChars = __charNumber;
			__textInput.restrict = "0-9a-z";
			this.addChild(__image);
			this.addChild(spacer);
			this.addChild(__textInput);
			__textInput.addEventListener(FlexEvent.DATA_CHANGE, __dataChanged);
		}
		
		/** Ce gestionnaire d'évènement est exécuté lorsque l'utilisateur modifie le texte saisi.
		 *  @param event Cet évènement est envoyé lorsque le texte saisi par l'utilisateur est modifié.
		 *  
		 */
		
		private function __dataChanged(event:Event):void
		{
			__onModifHd();
		}
		
		/** Cette méthode permet de nommer le composant. Le nom est utilisé pour désigner l'émetteur d'un évènement de changement d'état fonctionnel.
		 *  @param inName Nom du composant.
		 */
		
		public function setName(inName:String):void
		{
			__myName = inName;
		}
		
		/** Cette méthode libère les ressources allouées au composant.
		 */		
		
		public function destroy():void
		{
			__textInput.removeEventListener(FlexEvent.DATA_CHANGE, __dataChanged);
		}
		
		/** Cette méthode renvoie les valeurs sélectionnées via le composant.
		 *  @return La méthode renvoie un objet qui présente les propriétés suivantes :
		 *          o 'text': La chaîne de caractères entrée par l'utilisateur.
		 *          o 'length': Le nombre de caractères entrés par l'utilisateur.
		 *          o 'session': La clé de session associée à l'image.
		 */
		
		public function getSelectedValues():Object
		{
			return { text:__textInput.text, session:__session, length:__textInput.length };
		}
		
		/** Dans le cas particulier de ce composant, cette méthode n'est pas utilisée.
		 *  Elle est néanmoins présente afin d'implémenter l'interface "ISelectorInterface".
		 */
		
		public function setValues(inValues:Object, inDispatchEvt:Boolean=true):void
		{
		}
		
		/** Cette méthode rend le composant utilisable par l'utilisateur.
		 */
		
		public function enable():void
		{
			__textInput.enabled = true;
		}
		
		/** Cette méthode rend le composant inutilisable par l'utilisateur.
		 */
		
		public function disable():void
		{
			__textInput.enabled = false;
		}
		
		/** Cette méthode réinitialise le composant.
		 */
		
		public function reset(inValues:Object=null):void
		{
			__textInput.text = '';
		}
		
		/** Cette méthode permet de définir une fonction, qui sera exécutée par l'acteur, à chaque fois que ce dernier change d'état.
		 *  @param inFunction Fonction exécutée par le composant à chaque fois que ce dernier change d'état.
		 */
		
		public function setOnModificationHandler(inFunction:Function=null):void
		{
			if (inFunction != null) { __onModifHd = inFunction; }
		}
		
	}
}