package tools
{
	import components.popups.HttpError;
	import components.popups.ServerError;
	import components.popups.WaitBox;
	
	import flash.display.DisplayObject;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	
	import gestionnaires.EventHandlerControler;
	
	import interfaces.IGhostInterface;
	
	import mx.core.IFlexDisplayObject;
	import mx.managers.PopUpManager;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.HTTPService;

	/** Cette classe implémente un service de téléchargement (séquentiel). Le composant peut-être configuré de deux façons :
	 *         Comportement "synchrone". Une boîte modale va bloquer l'interface utilisateur tant que le traitement ne sera pas entièrement terminé.
	 *         Comportement "asynchrone". Les interactions entre l'utilisateur et l'application sont permises.
	 * 
	 *  Il est également possible de déclencher l'exécution automatique d'une fonction à la fin du téléchargement.
	 *  Cette fonction peut être utilisée pour modifier le document téléchargé, afin de l'adapter à une utilisation spécifique.
	 *  Ou bien, cette fonction peut-être exécutée pour effectuer tous types d'actions.
	 * 
	 *  Remarque extraite de la documentation de FLEX :
	 *  Although you can load data into your application from other domains, these other domains must implement
	 *  an open crossdomain.xml file or add the domain that your SWF file is loaded from to their crossdomain.xml
	 *  file.
	 * 
	 *  @warning Il faut être prudent lors de l'utilisation des fonctions getDocument() et error().
	 *           Lorsque la fonction load() retourne, le document n'est pas encore téléchargé!!!
	 *           Pour exécuter une action dès que le document est téléchargé, il faut utiliser un greffon.
	 * 
	 *  @remark Ce composant présente la particularité de procéder à des téléchargements séquentiellement.
	 *          Si plusieurs téléchargements sont demandés, ils seront effectués l'un après l'autre.
	 *          Ce comportement est obtenu grâce à l'utilisation d'une propiété statique.
	 * 
	 *  @remark La machine virtuelle Flash ne fait qu'une seule chose : Exécuter des gestionnaires d'évènement, au grès des évènements.
	 *          L'exécution d'un gestionnaire d'évènement ne peut pas être interrompue (par un évènement).
	 *          Autrement dit, un gestionnaire d'évènement s'exécute de façon séquentielle, sans interruption.
	 * 
	 *  @note Amélioration possible : Autoriser les téléchargements simultanés depuis des sources distinctes.
	 */
	
	[ Event( name="LoaderEvt", type="HttpLoaderEvt") ]
	public class HttpLoader extends EventDispatcher implements IGhostInterface
	{
		private var __evtHd:EventHandlerControler = new EventHandlerControler();
		private var __result:Object               = null;
		private var __error:Boolean               = false;
		private var __plugin:Function             = null;
		private var __synchro:Boolean             = true;
		private var __http:HTTPService            = new HTTPService();
		private var __popup:IFlexDisplayObject;
		private var __parent:DisplayObject;

		/** File d'attente de téléchargement, commune à tous les objets de téléchargement.
		 */

		static private var __downloads:Array = null;

		/** Crée un chargeur de document via HTTP.
		 *  @param inParent Ce paramètre définit le composant d'affichage utilisé pour attacher, les menus surgissant.
		 */
		
		public function HttpLoader(inParent:DisplayObject, target:EventDispatcher=null)
		{
			super(target);
			
			// On s'assure que la file de téléchargements est instanciée une seule fois!
			if (__downloads == null) { __downloads = new Array(); }
			__parent = inParent;
			__setEventHandler();
		}
		
		/** Cette méthode est utilisée pour déclarer les évènements traités par cette classe.
		 */	

		private function __setEventHandler():void
		{
			__evtHd.addEventHandler(__http, FaultEvent.FAULT, __httpErrorPopup);
			__evtHd.addEventHandler(this,   'LoaderEvt',      __loadNow);
		}
		
		/** Cette méthode lance le prochain téléchargement dans la liste des téléchargements planifiés.
		 *  @param event Cet évènement est envoyé pour déclencher un téléchargement.
		 */
		
		private function __loadNow(event:HttpLoaderEvt):void
		{
			var values:Object;
		
			trace(' ');
			trace ('HTTPLoader:__loadnow(): action = ' + event.getAction());
			trace ('HTTPLoader:__loadnow(): Download stack size is ' + __downloads.length);
			// Si la liste des téléchargements planifiés est vide, on ne fait rien.
			if (__downloads.length == 0) { return; }
		
			// On lance le prochain téléchargement planifié.
			values = __downloads[0];
			trace ('HTTPLoader:__loadnow(): Loading URL ' + values.url);
			__error   = false;
			__result  = null;
			__plugin  = values.plugin;
			__synchro = values.synchro;
			
			// Affichage de la boîte modale qui oblige l'utilisateur à attendre la fin de l'opération.
			if (__synchro)
			{
				__popup = PopUpManager.createPopUp(__parent, components.popups.WaitBox, true);
				PopUpManager.centerPopUp(__popup);
			}
			
			// On envoie au serveur la demande de la liste des codes postaux.
			__evtHd.addEventHandler(__http, ResultEvent.RESULT, __processLoadedDocument);
			__http.url          = values.url;  
			__http.method       = values.method;
			__http.resultFormat = values.resultFormat;
			__http.send(values.values);
		}
		
		/** Cette méthode planifie le téléchargement d'un document.
		 *  Le téléchargement n'est pas lancé immédiatement : Un évènement est envoyé pour demander le téléchargement.
		 *  Si un évènement de téléchargement est déjà en cours de traitement, l'évènement sera placé en file d'attente.
		 *  De cette façon, les téléchargements sont séquentiels.
		 *  @param inUrl URL vers le document à télécharger.
		 *  @param inPlugin Fonction à exécuter lorsque le document a été chargé.
		 *         Cette fonction doit accepter un paramètre, le document chargé.
		 *         Et il doit renvoyer un objet (éventuellement, la valeur null).
		 *         Prototype : function plugin(inValue:Object):Object.
		 *         Si la valeur renvoyée est différente de la valeur null, alors la valeur renvoyée sera affectée au document téléchargé.
		 *         Pour récupérer le "nouveau" document, on pourra appeler les fonctions suivantes : error(), suivi de getDocument().
		 *  @param inFormat Ce paramètre indique sous quelle forme doit être présenté le document téléchargé.
		 *         Pour connaître les valeurs possibles, veuillez consulter la documentation de la classe "HTTPService" (propriété "resultFormat").
		 *  @param inSyncronous Ce paramètre permet de spécifier le comportement du chargeur.
		 *         La valeur "true" indique que le chargeur est "synchrone". Une boîte modale va bloquer l'interface utilisateur tant que le traitement ne sera pas entièrement terminé.
		 *         La valeur "false" indique que le chargeur est "asynchrone". Les interactions entre l'utilisateur et l'application sont permises.
		 *  @param inValues Cet objet contient une liste de valeurs à envoyer au serveur, dans la requête HTTP.
		 *  @param inMethod Méhode utilisée ("GET" ou "POST").
		 */
		
		public function load(inUrl:String, inPlugin:Function=null, inFormat:String='text', inSyncronous:Boolean=true, inValues:Object=null, inMethod:String='GET'):void
		{
			trace(' ');
			trace('HTTPLoader:load() ' + inUrl);
			
			__downloads.push( {	url:inUrl,
								method:inMethod,
								resultFormat:inFormat,
								values:inValues,
								plugin:inPlugin,
								synchro:inSyncronous } );
			
			trace('HTTPLoader:load() Download stack size is ' + __downloads.length);
			// On envoie un évènement pour démarrer le premier téléchargement.
			if (__downloads.length == 1)
			{
				trace('HTTPLoader:load() First download => Disparch event [NewDownload]');
				dispatchEvent(new HttpLoaderEvt('LoaderEvt', this, "NewDownload", true));
			}
		}

		/** Ce gestionnaire d'évènement est appelé lorsque le composant HTTP reçoit la réponse du serveur.
		 *  @param event Évènement lancé par le service HTTP lorsque la réponse du serveur est reçue.
		 */

		private function __processLoadedDocument(event:Event):void
		{
			var popup:IFlexDisplayObject;
			var res:Object;
		
			trace(' ');
			trace ('HTTPLoader:__processLoadedDocument(): Got the document!');
			
			// On commence par retirer ce gestionnaire d'évènement.
			__evtHd.removeEventHandler(__http, ResultEvent.RESULT);
			
			// On extrait la réponse du serveur.
			try	{ __result = __http.lastResult; }
			catch (err:Error)
			{
				__error  = true;
				trace (err.message);
				if (__synchro) { PopUpManager.removePopUp(__popup); }
				popup = PopUpManager.createPopUp(__parent, components.popups.ServerError, true);
				PopUpManager.centerPopUp(popup);
			}
			
			// Il faut libérer les ressources.
			__http.clearResult();

			// On exécute le greffon, s'il est défini.
			if ((__plugin != null) && (! __error)) { res = __plugin(__result); if (res != null) { __result = res; /* trace ("HttpLoader res = " + res.toString()); trace('Value updated'); */ }; }
			if ((! __error) && __synchro) { PopUpManager.removePopUp(__popup); }
			
			// Le téléchargement a réussi.
			// On dépile le téléchargement qui vient de se terminer.
			// Puis, on lance le prochain dans la liste.
			trace ('HTTPLoader:__processLoadedDocument(): Dépile un téléchargement');
			__downloads.shift();
			dispatchEvent(new HttpLoaderEvt('LoaderEvt', this, "DownloadSucceed", true));
		}

		/** Cette méthode rend le composant éligible à la destruction.
		 */		
		
		public function destroy():void
		{
			__evtHd.removeAllEventHandlers();
		}
		
		/** Ce gestionnaire d'évènement est utilisé pour signaler les erreurs HTTP.
		 *  @param event Évènement lancé par le service HTTP lorsqu'une erreur se produit.
		 */

		private function __httpErrorPopup(event:Event):void
		{
			var popup:IFlexDisplayObject;
			trace(' ');
			trace ("Got an error!");
			if (__synchro) { PopUpManager.removePopUp(__popup); }
			popup = PopUpManager.createPopUp(__parent, components.popups.HttpError, true);
			PopUpManager.centerPopUp(popup);
			__evtHd.removeEventHandler(__http, ResultEvent.RESULT);
			
			// Le téléchargement a échoué.
			// On dépile le téléchargement qui vient de se terminer.
			// Puis, on lance le prochain dans la liste.
			__downloads.shift();
			dispatchEvent(new HttpLoaderEvt('LoaderEvt', this, "DownloadFailed", true));
		}
		
		/** Cette méthode renvoie le document téléchargé.
		 *  @return La méthode renvoie un objet qui contient le document téléchargé.
		 *  @warning Veillez à ne pas appeler cette méthode avant que le document ne soit chargé!
		 */

		public function getDocument():Object { return __result; }
		
		/** Cette méthode permet de savoir si une erreur s'est produite.
		 *  @return Si une erreur est survenue, la méthode renvoie la valeur "true".
		 *          Sinon, la méthode renvoie la valeur "false".
		 *  @warning Veillez à ne pas appeler cette méthode avant que le document ne soit chargé!
		 */
		
		public function error():Boolean { return __error; }
	}
	
	
	
}

import flash.events.Event;

/** Cette classe définit un évènement spécifique aux objets de la classe "HTTPLoader".
 *  @remark Cette classe est une classe interne. Cela signifie qu'elle n'est accessible que depuis les classes définies dans ce fichier.
 */

internal class HttpLoaderEvt extends Event
{
		private var __booleanBubbles:Boolean;
		private var __booleanCancelable:Boolean;
		private var __source:Object;
		private var __action:String;
		private var __stringType:String;
	
		/** Crée un évènement de type "HttpLoaderEvt".
		 *  @param source Cet objet représente le lanceur d'évènement.
		 *  @param action Cette chaîne représente le type d'action signalé par cet évènement.
		 */
	
		public function HttpLoaderEvt(type:String, source:Object, action:String, bubbles:Boolean=false, cancelable:Boolean=false)
		{
			super(type, bubbles, cancelable);
			__stringType        = type;
			__booleanBubbles    = bubbles;
			__booleanCancelable = cancelable;
			__source            = source;
			__action            = action;
		}

		override public function clone():Event
		{
			return new HttpLoaderEvt(__stringType, __source, __action, __booleanBubbles, __booleanCancelable);
		}
		
		public function getAction():String { return __action; }
		
		public function getSource():Object { return __source; }
}

