package tools
{
	import flash.events.Event;
	
	import mx.core.Container;
	import mx.effects.AddChildAction;
	import mx.effects.Effect;
	import mx.effects.RemoveChildAction;
	import mx.effects.Sequence;
	import mx.events.EffectEvent;
	
	/** Cette classe permet de créer des transitions entre des états, sur une scène.
	 *  Un état se caractérise par l'affichage d'un objet dans un conteneur (la scène).
	 *  Une transition de l'état "E1" vers l'état "E2" se caractérise par :
	 *  o La disparition de l'objet associé à l'état "E1".
	 *  o L'apparition de l'objet associé à l'état "E2".
	 *  Note : La disparition d'un objet de la scène est associée à une transformation.
	 *         Une transformation est une suite d'effets (listés dans un tableau). 
	 */
	
	public class TransitionMaker
	{
		private var __scene:Container;
		private var __currentState:String;
		private var __nextState:String;
		private var __statesList:Array;
		private var __sequence:Sequence;
		private var __statesIndex:Object = new Object();
		private var __busy:Boolean       = false;
		
		/** Crée un objet de gestion des transitions.
		 *  @param inScene Composant sur lequel sont jouées les transitions.
		 *  @param inStates Ce tableau contient des éléments de type "Object".
		 *         Chaque élément représente un état.
		 *         Un état, dans le contexte de cette classe, est associé à un composant affiché sur la scène.
		 *         Le passage entre deux états successifs définit une transition.
		 *         Une transition correspond à la disparition du composant courant et à l'apparition du composant associé au nouvel état.
		 *         Les propriétés des objets qui représentent les éléments du tableau sont :
		 *         o name: Le nom de l'état (qui correspond à l'affichage d'un composant sur la scène).
		 *         o display: Objet d'affichage qui caractérise l'état.
		 *         o appear:  Cet attribut est un tableau d'objets de types "Effect".
		 *                    Les effets placés dans le tableau seront joués, les un après les autres, lors de l'apparition du composant. 
		 *         o disappear: Cet attribut est un tableau d'objets de types "Effect". 
		 *                      Les effets placés dans le tableau seront joués, les un après les autres, lors de la disparition du composant.
		 *  @param inCurrentState Nom de l'état courant (par défaut, le premier de la liste).
		 */
		
		public function TransitionMaker(inScene:Container, inStates:Array, inCurrentState:String=null)
		{
			if (inStates.length < 2) { throw new Error('The list of states contains ' + inStates.length + ' elements! (it should contain, at least, 2 elements)'); }
			__statesList = inStates;
			__scene      = inScene;
			if (inCurrentState == null) { __currentState = inStates[0].name; } else { __currentState = inCurrentState; }
			__createIndex();
		}
		
		/** Cette fonction crée un index de la liste des états.
		 *  L'index associe les noms des états à leurs positions dans la liste des états.
		 */
		
		private function __createIndex():void
		{
			var name:String;
			
			for (var i:Number=0; i<__statesList.length; i++)
			{
				name = __statesList[i].name;
				if (__statesIndex.hasOwnProperty(name)) { throw new Error('Duplicate state name ' + name); }
				__statesIndex[name] = i;
			}
			if (! __statesIndex.hasOwnProperty(__currentState)) { throw new Error('Unknown initial state: ' + __currentState); }
		}
		
		/** Cette méthode fait passer la scène d'un état à un autre.
		 *  o La séquence de dispariation de l'état courant est joué sur le composant présent sur la scène.
		 *  o Le composant présent sur la scène disparaît.
		 *  o Le composant associé à l'état de destination apparaît sur la scène.
		 *  o La séquence d'apparition de l'état courant est joué sur le composant qui vient d'apparaître.
		 *  @param inName Nom de l'état de destination.
		 */
		
		public function goToState(inName:String):Boolean
		{
			var currentObject:Object;
			var nextObject:Object;
			var addAction:AddChildAction;
			var removeAction:RemoveChildAction;
			var e:Effect;

			if (__busy)                   { trace ('Rejected - I am busy!'); return false; }
			if (inName == __currentState) { trace ('Rejection - Same destination state!'); return false; }

			__busy       = true;	
			__nextState  = inName;
			__sequence   = new Sequence();
			
			currentObject = __getStateByName(__currentState);
			nextObject    = __getStateByName(__nextState);
			removeAction  = new RemoveChildAction(currentObject.display);
			addAction     = new AddChildAction(nextObject.display);
			addAction.relativeTo = __scene;
						
			// Construction de la transformation.
			for each (e in currentObject.disappear) { e.target = currentObject.display; __sequence.addChild(e); }
			__sequence.addChild(removeAction);
			__sequence.addChild(addAction);
			for each (e in nextObject.appear) { e.target = nextObject.display; __sequence.addChild(e); }
			
			// Activation de la transformation.
			__sequence.addEventListener(EffectEvent.EFFECT_END, __sequenceDone);
			__sequence.play();
			return true;
		}

		/** Cette fonction renvoie l'index d'un état, désigné par son nom, dans la liste des états.
		 *  @param inName Nom de l'état.
		 */

		private function __getStateByName(inName:String):Object
		{
			return  __statesList[__statesIndex[inName]];
		}
		
		/** Ce gestionnaire d'évènement est exécuté lorsque la séquence qui s'applique à la scène est terminée. 
		 *  @param event Évènement envoyé lorsque la séquence qui s'applique à la scène est terminée. 
		 */
		
		private function __sequenceDone(event:Event):void
		{
			trace ('Got event!');
			__sequence.removeEventListener(EffectEvent.EFFECT_END, __sequenceDone);
			__busy     = false;
			__sequence = null;
		}
		
	}
}

